/* ================================================================
   routinen zum Scannen und Ueberpruefen von strings

   (c) Heisch Automation, Werner Heisch
       http://sites.inka.de/heisch

   ================================================================
   Stand  21.02.2009 
   Stand  29.01.2009 Start


!!! rules are not implemented yet !


Funktion:
Uebergebene Strings werden in token zerlegt, dabei werden die Separatoren 
unterschieden zwischen:
gsep = allgemeine Separatoren
        - allgemeine Separatoren werden nicht registriert,
        - am Stringanfang und Stringende gefundene allgemeine Separatoren 
          werden eliminiert
csep  = Separatoren, die aus einem Character bestehen

ssep  = Separatoren, die aus Strings bestehen.
        Spezielles: Wildcard Endung
        Wenn der Separator mit '*' endet sind alle Worte mit gleichem Anfang
        ein Treffer. Das gesuchte Wort endet dann am nächsten GSEP oder CSEP
        Soll ein Wort exakt treffen, der letzt Buchstabe ist aber ein '*', so muss
        als Separator-Ende '**' geschrieben werden.
        Wenn exakt getroffen werden soll, muss das dem gefundenen Wort folgende
        Zeichen ein GSEP oder CSEP sein.

rule  = derzeit noch nicht realisiert.

Zur Verarbeitung:

Es muß eine apsan-structure mit absan_gen() erzeugt werden.
Alle weitere Operatioen laufen mit Hilfe dieser structure 

Der eingelesene Original-String wird in "obuf" gespeichert,
Beim Einlesen wird der String entsprechend ueber die 
Separatorenden definierten Regeln zerlegt. 
Bei der Zerlegung wird der Original-String auf weitere Buffer verteilt
und darauf werden pointer gesetzt, so dass token und Separatoren
am Ende als Strings vorliegen, die über pointer zugaenglich sind.

Token und Separatoren (csep und ssep, nicht gsep) werden abwechselnd in
buf[0] und buf[1] abgelegt. damit jeweils hinter einem Eintrag eine Luecke 
entsteht, um den String mit '\0" zu terminieren.

*/

#include<sys/types.h>




#define APSAN_NONE 0
#define APSAN_TOKEN 1 
#define APSAN_GSEP 2 
#define APSAN_CSEP 4
#define APSAN_SSEP 8
#define APSAN_RULE 16


#define APSAN_MAXPATTERN 100  /* maximum 100 pattern in a rule-definition */


#define APSAN_isalnum 1
#define APSAN_isalpha 2
#define APSAN_isascii 4
#define APSAN_isblank 8
#define APSAN_iscntrl 16
#define APSAN_isdigit 32
#define APSAN_isgraph 64
#define APSAN_islower 128
#define APSAN_isprint 256
#define APSAN_ispunct 512
#define APSAN_isspace 1064
#define APSAN_isupper 2048
#define APSAN_isxdigit 4096

#define APSAN_MAX_SETS 100 /* maximum cout of type sets */

/* ---------------- LIST OF ERROR CODES --------------- */
/*
  0 = success
 -1 = no space left to insert s
 -2 = unknown type
 -3 = no space left to increase sptr -structure
 -4 = no space left to increase rule -structure
 -5 = too much tokens, need higher "max_toks" in "apsan_gen()".
 -6 = requested type set is not defined
 -7 = number of typeset is out of range [0 ..APSAN_MAX_SETS] 
 -8 = no space left to genereate a new type set
 -9 = this type of separators (GSEP,CSEP) not allowed here.
-10 = size of seperator definition = 0 
-11 = apsan_rstrT() strt_idx out of range
-12 = apsan_rstrT() strt_how out of range
-13 = apsan_rstrT() end_idx out of range
-14 = apsan_rstrT() end_how out of range
*/

/* ---------------------------------------------------- */
/* Ptr to te central data structure of APSAN
   ( That's all the user application program needs to know )
*/

typedef struct apsan_stru *APSAN_P;

/* ---------------------------------------------------- */
/* structure to use  apsan_def_id()
*/

typedef struct apsan_def_id {
  int id;		/* id number */
  char *str;		/* according string */
}  Apsan_def_id ;

/* ---------------------------------------------------- */
/* structure for the reported results
*/
/* this structure describes also internal informations about found tokens */
typedef struct apsan_reg_type {
  char *text;             /* Pointer to the token or separator */
  unsigned char type;    /* Type of token, where ptr is pointing to:
                            NONE = 0, TOKEN = 1 GSEP = 2 (spare), CSEP = 4 
                            RULE = 16 */
  int id;               /* if CSEP or SSEP or RULE, then number according to list,
                            defined by cptr or sptr or rptr */
  int strt;              /* index start position in obuf */
  int end;		 /* index end position in obuf (points to last char */
} Apsan_reg_type;


/* ---------------------------------------------------- */
/* structure for the reported results
*/
typedef struct apsan_result_type *apsan_result_p;


typedef struct apsan_result_type {
  int cnt;                 /* count of matched tokens and separators */
  struct apsan_reg_type *r; /* copy of the internal used reg[] structure */
}  Apsan_result;


/* =====================================================*/
/* headers of the public functions */

/* ---------------------------------------------------- */

/* generate an new apsan-structure and initialize the data

Parameters : astru     = pointer to a apsan-structure
             data_len  = maximum length of data in bytes
             max_toks  = maximum count of tokens of the input line,
                        including possible separators CSEP and SSEP
             max_gsep  = maximum count of general separators
                        space will be allocated

             Also the first type set will be allocated and initialized:
             max_csep_0  = maximum count of character separators
                        space will be allocated
             max_ssep_0  = maximum count of string separators
                         only space for organisation will be allocated
             max_rules_0  = maximum count of rules
                         only space for organisation will be allocated

on return: 0 = structure is successfully initialized
               a pointer to the initialized structure is returned,
          -1 = no data left to initialize
               no pointer to structure is returned.

/* ------------ */

int apsan_gen(APSAN_P *astru,size_t data_len,int max_toks,\
    size_t max_gsep, size_t max_csep_0, int max_ssep_0, int max_rules_0) ;

/* ---------------------------------------------------- */
/* close  a formally opened apsan structure

Parameters : astru     = pointer to a apsan-structure


on return: 0 = structure is successfully initialized
               a pointer to the initialized structure is returned,
          -1 = no data left to initialize
               no pointer to structure is returned.

/* ------------ */
void apsan_end(APSAN_P astru) ;

/* ---------------------------------------------------- */
/* define separators

Parameters : astru     = pointer to a apsan-structure
                         (defined before with apsan_gen() )
             type      = APSAN_GSEP  global separators
                         APSAN_CSEP character separators
                         APSAN SSEP string separators

             ts_no     = if type == APSAN_GSEP : dont care
                         else: number of the type set, to use.
                         range: 0 .. APSAN_MAX_SETS


             s         = string with definitions
                         The string separators are separated by ',', if ','
                         shall be treated as a sign, it shall be written twice; ',,'
                         APSAN RULE rule set
                         The rules are separated by ',', if ','
                         shall be treated as a sign, it shall be written twice; ',,'
                         rule syntax : 
                         Ending a token with '*' represents all signs, if '*' only shall 
                         be treated as a single char, it has to be written trice.

                         Special type checking tokens:
                         conditions: preceeding and following char must be a GSEP or CSEP 
                         :isint: checks a token against an integer including the
                                 optional preceeding sign
                         :isfloat: checks a token against a float including the
                                 optional preceeding sign.

on return: 0 = success
          -1 = no space left to insert s
          -2 = unknown type
          -3 = no space left to increase sptr -structure
          -4 = no space left to increase rule -structure
          -7 = number of typeset is out of range [0 ..APSAN_MAX_SETS] 
          -8 = no space left to genereate a new type set


/* ------------ */
int apsan_def(APSAN_P astru, int type,int ts_no, const char s[]) ;

/* ---------------------------------------------------- */
/* define separators SSEP,RULES within a list with 
   user defined id  numbers.
This function aspects an array [1..cnt] of structures of the form a 
  struct { int id1, char[..]"name_1"
           int id2, char[..]"name_2"
         }
  where id must be from type integer 
  and the string can by of any lenght longer than 0, with any contents, ending
  with '\0'.

see also : apsan_def()
Using apsan_def(), the SSEP and RULE separators are given in a single string, 
separated by ','id numbers are auto generated, beginning with 1.

Using this function,the id numbers are defined individually.

If a formerly used definition will be overwritten, additionaly needed space will 
be attached, but remaining space will never be freed. 

Parameters : astru     = pointer to a apsan-structure
                         (defined before with apsan_gen() )
             type      = APSAN SSEP string separators
                         The string separators are separated by ',', if ','
                         shall be treated as a sign, it shall be written twice; ',,'
                         APSAN RULE rule set
                         The rules are separated by ',', if ','
                         shall be treated as a sign, it shall be written twice; ',,'
                         rule syntax : 
                         * represents all signs
             ts_no     = number of the type set, to use.
                         range: 0 .. APSAN_MAX_SETS

             s          = pointer to the structure of separators declared by Apsan_def_id
                         The string separators are separated by ',', if ','
                         shall be treated as a sign, it shall be written twice; ',,'
                         APSAN RULE rule set
                         The rules are separated by ',', if ','
                         shall be treated as a sign, it shall be written twice; ',,'
                         rule syntax : 
                         Ending a token with '*' represents all signs, if '*' only shall 
                         be treated as a single char, it has to be written trice.

                         Special type checking tokens:
                         conditions: preceeding and following char must be a GSEP or CSEP 
                         :isint: checks a token against an integer including the
                                 optional preceeding sign
                         :isfloat: checks a token against a float including the
                                 optional preceeding sign.

             cnt        = count of array elements



on return: 0 = success
          -1 = no space left to insert s
          -2 = unknown type
          -3 = no space left to increase sptr -structure
          -4 = no space left to increase rule -structure
          -7 = number of typeset is out of range [0 ..APSAN_MAX_SETS] 
          -8 = no space left to genereate a new type set
          -9 = this type of separators (GSEP,CSEP ) not allowed here.


/* ------------ */

int apsan_def_id(APSAN_P astru, int type,int ts_no, Apsan_def_id *s,int cnt);

/* ---------------------------------------------------- */
/* load a string and interpret

Parameters : astru     = pointer to a apsan-structure
                         (defined before with apsan_gen() )
             reg_what  = a mask, what to register and to count.
                         !!! tokens and defined GSEOPs, CSEPs and SSEPs are always
                         !!! 'reg_what' only decides, if the are registered in reg. 
                         Possible values:
                         APSAN_NONE 0 = only token are registered
                         if 'reg_what' is set to non zero ( not 0 or APSAN_NONE )
                         APSAN_TOKEN must be set to be part of the result.
                         APSAN_TOKEN 1 = only token
                         APSAN_CSEP  4 = token and cseps
                         APSAN_SSEP  8 = token and sseps
                         APSAN_CSEP | APSAN_SSEP  12 = token,cseps and sseps
             ts_no     = number of the type set, to use.
                         if none, use -1 
                         else 0 .. APSAN_MAX_SETS

             s         = string to investigate
                         If set to NULL, the old string is evaluated again,
                         maybe with other SEPs od other 'reg_what'

on return: 0 >= success, count of cound token and separators SSEP and CSEP type
          -1 = buffers too short to insert s
               need higher "data_len" in "apsan_gen()".
          -5 = too much tokens, need higher "max_toks" in "apsan_gen()".
          -6 = requested type set is not defined
          -7 = number of typeset is out of range [-1 ..APSAN_MAX_SETS] 

Evaluation:
GSEPs are not registered in *reg.
First, the gseps at the end of the file are deleted, then the gseps 
at the start. 
Then the resulting inner part is evaluated:
  presedence: RULE-> GSEP -> SSEP -> CSEP
Subsequential GSEPs are registered as one, all others are registered
by their occurance.

/* ------------ */
int apsan_scan(APSAN_P astru,int reg_what,int ts_no, const char s[]) ;

/* ---------------------------------------------------- */
/* print the structure contents
( for trouble shooting only)

/* ---------------------------------------------------- */
/* report the result of last scan

Parameters : astru     = pointer to a apsan-structure
                         (defined before with apsan_gen() )

on return: NULL ==  no result found, no formaerly scan ?
                else : Ptr to structure Apsan_result

/* ------------ */
apsan_result_p apsan_result(APSAN_P astru);

/* ---------------------------------------------------- */
/* report a string, part of the incoming line
   the resulting string is limited by '\0'

Parameters : astru     = pointer to a apsan-structure
                         (defined before with apsan_gen() )
             str       = string to be filled
             strt      = Index, start -position of the string
             end       = end position of the string 
                         OR 0 = return string up to the end last token != GSEP
                         OR -1 = return up to the complete endof input string,
                                 including all GSEP at the end of incoming string  
on return: (char *) to str

/* ------------ */
char *apsan_rstr(APSAN_P astru, char *str, int strt, int end);

/* ---------------------------------------------------- */
/* report a string, part of the incoming line
   start and end are defined by pointing to structure created by 
   the last apsan_scan() (and reported to the user by apsan_result().).
   the resulting string is limited by '\0'

   !! strt_idx and end_idx are starting at 0 !

Parameters : astru     = pointer to a apsan-structure
                         (defined before with apsan_gen() )
             str       = string to be filled

             strt_idx  = index of the token to be used as start:
                          0 .. n : nr of found token  according to 
                                   apsan_result()
             strt_how  =  0  = Start of next token behind this token
                          1  = including the token, pointed to
                          2  = next char behind the token, including GSEP + CSEP

             end_idx   =  index of the token to be used as End:
                          0 .. n : nr of found token according to 
                                   apsan_result()
             end_how   =  0  = End of last token before this token
                          1  = including the token, pointed to
                          2  = last char before the token including GSEP + CSEP


on return: (char *) to str
                    or NULL, if an error occured. 
                    The reason for the error is storted in a->errno

/* ------------ */

char *apsan_rstrT(APSAN_P astru, char *str, 
                 int strt_idx, int strt_how,
                 int end_idx,  int end_how ) ;

/* ---------------------------------------------------- */
/* report a token, number according to reported to the user 
   by apsan_result().).
   the resulting string is limited by '\0'

   !! strt_idx and end_idx are starting at 0 !

Parameters : astru     = pointer to a apsan-structure
                         (defined before with apsan_gen() )
             str       = string to be filled

             idx  = index of the token

on return: (char *) to str
                    or NULL, if an error occured. 
                    The reason for the error is storted in a->errno

/* ------------ */

char *apsan_rTok (APSAN_P astru, char *str,int idx );


/* ==================================================== */

/* for analysis purposes
*/

/* ------------ */
/* dump the contents of the structures */

void apsan_dump(APSAN_P astru) ;

/* ------------ */
/* print the result of the last scan */

void apsan_print_result( APSAN_P s) ;

