/* -----------------------------------

     p2_data.h

 (c) Heisch Automatisierungstechnik, Werner Heisch
       hw@heisch.inka.de
       http://sites.inka.de/heisch 

 Stand
   19.02.2009  Pult 2
   18.02.2001  CPU-Nr fuer MPI-Adapter 
   15.07.2000  HW Einbau Test-Funktion 


/* ---------- defines and type definitions -------- */

// #include <sys/types.h>
#include <stdio.h>
#include <time.h>

#define ON 1
#define OFF 0

#define OPMODE_NONE 0
#define OPMODE_PROG 1
#define OPMODE_RUN  2
#define OPMODE_TEST 3

#define A_ANZ 128
#define E_ANZ 48
#define EW_ANZ 18
#define AW_ANZ 18
#define PA_ANZ 20  /* Anzahl Prozessabblider */
#define KDO_ANZ 10 /* Anzahl Kommandos */

#define V_TYPE_E   1 /* type E */
#define V_TYPE_A   2 /* type A */
#define V_TYPE_EW  3 /* type EW */
#define V_TYPE_AW  4 /* type AW */

#define INACTIVE 1
#define KF16 2
#define KH16 3
#define KM16 4
#define KT16 5
#define KZ16 6
#define KF32 7
#define KH32 8
#define REAL 9
#define S5KG 10
#define KY   11
#define BIT1 101   /* Bit Set */
#define BIT0 102   /* Bit Reset */
#define BITT 103   /* Bit toggle */

#define KDO_DB_MODE 0 /*Datenübergabe mit 0 = Kommando,DB 1 = direkt */ 
/* definitionen fuer Variabenanzeigen */
#define V_ACTUAL 0     /* normal actualisation */
#define V_NEUTRAL 1    /* set neutral, print name */
#define V_TEST_OK 2    /* set test-positive-color */
#define V_TEST_NOK 3   /* set test-negative-color */
#define V_DEF 4        /* set new definition, print name */

#define V_NAME_LENGTH 40 /* lenght odf the name ov a variable */



/* ---------- defines and type definitions -------- */

typedef struct {
 int pa_nr; /* Prozessabbild -Nr  -1 = nicht definiert */
 int db_nr;
 int dw_nr; /* word or byte address, according to plc */
 short bit_nr;
 short format_type;
 short val;
 signed char error;
 char name[V_NAME_LENGTH];
} bit_def_typ;

typedef struct {
 int pa_nr; /* Prozessabblid -Nr -1 = nicht definiert */ 
 int db_nr;
 int dw_nr;  /* word or byte address, according to plc */
 short bit_nr;
 short format_type;
 int  lval;
 double val;
 char  cval[20];
 signed char error;
 char name[V_NAME_LENGTH];
} eaw_def_typ;

typedef struct {
 unsigned short db_nr; 
 short dw_nr; /* Anfangs-DW WORD Address S5 !! */
 int dw_end; /* letztes DW WORD Address S5 !! */
 int len;   /* Laenge in WORDS */
 unsigned short dw[512]; /* inhalt */
} pa_typ;

/* Commando-Struktur */
typedef struct {
 short k_type; /* Command type KF16 ....  */ 
 signed char bit_nr;
 unsigned short db_nr;
 short dw_nr;
 long lval;
 double val;
} kdo_typ;

/* ---------- data definitions -------------------- */

  /* Verbindung zur Simatic */


  unsigned short s5db[20];      /* Einlese-Array fuer Simatic */

  signed char simatic_type;            /* Simatic_typ s(5), s(7) */      
  signed char bytenr_pro_word;         /* Bytes pro word  ( s5 = 1 / s7 = 2 */ 

  int cpu_nr;                   /* cpu-nr */
  int access_mode;		/* 0 = Kommando-DB, 1 = direkt */

  int kdo_dbnr; 		/* Kommando-DB */ 
  int kdo_dwnr; 		/* Kommando-DW-Nummer, Anfang des Kommandoblocks in AG */

  int write_lstat;		/* Fetch command, last state */
  int read_lstat;		/* write command last state */

/* ---------- data definitions ----------------------- */

/* Dateimanagement */
char wdir[1024];   /* Arbeitsdirectory */

char home_dir_name[1024];  /* PWD */

char structfi_name_short[20];
char structfi_name[1024];
char struct_dir[1024];   /* Directory der structure-files */
signed char struct_defined;     /* 0 = not defined, 1 = Defined an OK,  -1 = error */


FILE *str_fi;

char rkrc_fi[1024];
char rkrc_dir[1024];
char rkrc_fi_short[256];
signed char rc_defined;       /* 0 = not defined, 1 = Defined an OK,  -1 = error */



/* FL_Dirlist *hdir; */
/* const hdir *hdptr; */
FL_Dirlist   *akt_dir;  /* Directoryliste */
int akt_dir_len;      /* Directorylaenge */
int akt_dir_idx;      /* Index(=ptr) in aktuelle Directoryliste */


/* forms-Definitionen */
/* ------------ Daten --------------- */
  FD_pult *fd_pult;
  FD_e_structure *fd_e_structure;
  FD_ew_structure *fd_ew_structure;
  FD_a_structure *fd_a_structure;
  FD_aw_structure *fd_aw_structure;
  FD_ew_input   *fd_ew_input;
  FD_kdo_db_structure *fd_kdo_db_structure;
  FD_version_dt *fd_version_dt;
  FD_help_dt  *fd_help_dt;
  FD_version_en *fd_version_en;
  FD_help_en  *fd_help_en;

/* form-Steuerung */
int pult_xretval;  /* Rueckkehrfreigabe aus unter-Fenster */
int structure_open; /* Struktur bereits geoeffnet */
int help_open;      /* hilfe-Struktur bereits geoeffnet */
int xretval;        /* Return-wert aus einer Editierfunktion */

/* Timer-Variable */
double v_timerval;    /* Variable fuer Timereinstellung */


/* ------------ */

int op_mode;		/* Betriebsart: PROG(0),RUN (1) TEST (2)*/
int op_mode_old;        /* Betriebsart: zuvor */
int test_stp;           /* Step control in Testmode */

int recalc_pa_req;	/* trigger the re-calculation of pa */

/* ----- Programmzugriff-Variablen -------- */

bit_def_typ e_def[E_ANZ]; /* Input to Simatic */
bit_def_typ a_def[A_ANZ];  /* Bits from Simatic */
eaw_def_typ ew_def[EW_ANZ];
eaw_def_typ aw_def[AW_ANZ];

int all_def_idx;           /* index zum Rückspeichern der Definitionen */
int all_def_type;	   /* TYPE zum Rückspeichern der Definitionen */

int p_error;		   /* Error number of Pult */
int rk_error;		   /* errors generated by rk*_server */

kdo_typ     kdo[KDO_ANZ];
int kdo_anz;


pa_typ  pa[PA_ANZ]; /* Prozessabbild */
int     pa_anz; /* Anzahl Prozessabbilder */


/* ---- Zeitmanagement -------- */

time_t akt_zeit;        /* aktuelle Uhreit */

