/* =================================================================
   Rechnerkopplung zu SIMATIC-S7 über die rk*_server

   data link to Simatic S7 using the rk*_servers

       Teil: Client-Funktionen zu den RK*_SERVERn
             Beschreibung und Header-File fuer rk512m_client.o
             Mehrserverfaehige Version

             Diese Datei beinhaltet die an die S7 angepassten Aufrufe.

             Die  "klassischen" Aufrufe (S5-kompatibel) finden Sie 
             weiterhin in "rk512_client.h".


       Part: client functions to RK*_SERVERs
             Description and header file for rk512m_client.o
             This version can connect to several servers at the
             same time.

             This file contains the calls adapted to S7 conditions.

             The "classic" calls are furthermore available in
             "rk512_client.h"


(c)	Heisch Automation
	Im Vorderen Grossthal 4
	76857 Albersweiler
        Phone +49 6345 /9496732
 
   ==================================================================

   Stand V 5.08
		03.06.2013 
		#include <sys/types.h> wg. Raspberry Pi

        	11.08.2012
                - Neue Funktion: nur S7-kommunikation:
		  aktuelle Zeit an CPU (DB) übertragen 
		- rk_svr_inf_type erweitert für Online-Diagnose

   Stand V 5.01
        	09.03.2009 
                Neue Fehlermeldungen wg. S7-Kommunikation
                neue Groesse des Shared Memory

		rk7_* .. Aufrufe


   ==================================================================

Inhalt:
-------
Diese Datei enthaelt die Beschreibung der Funktionsaufrufe mit ihren
Parametern und den zurueckgelieferten Fehler-Codes.

Gleichzeitig dient sie als Header-File fuer rk512_client.o.

Content:
-------
This file contains the description of the function calls including their
parameters and the error codes they return.

At the same time it is the Header-File of rk512_client.o.
*/

/* --------------- Funktionsaufrufe --------------------- */

/*  Mehr-Server-faehige Version

  Diese Version der Client-Library erlaubt es, auf mehrere Server
gleichzeitig zuzugreifen. Zu diesem Zweck haben alle Aufrufe als
ersten Parameter eine sogenannte Servernummer.
Moegliche Servernummern sind die Zahlen 0 bis 9. Dies duerfte ausreichen.

Die Funktion rk7m_open_shm ordnet der Servernummer einen Server
zu. Die restlichen Funktionen koennen durch Angabe der Servernummer
auf einen beliebigen, mit rk7m_open_shm geoeffneten Server zugreifen.

Namensgebung:
  Die Funktionen, die als ersten Parameter die Servernummer erwarten,
haben vor dem ersten Unterstrich im Namen ein 'm' (steht fuer mehrserver).
Alle diese Funktionen gibt es auch ohne dieses 'm' und ohne den
Parameter server. Sie beziehen sich auf den Server mit der Nummer STDSVR.
Die Aufrufe fuer den STDSVR sind als MACRCOs implementiert.


--------------------------

Multi-Server-Version

  This version of the Client-Library allows to connect to several Servers
at the same time. For this purpose all calls use as their frist argument
the server number. Server numbers range from 0 to 9 inclusive.
Using the function rk7m_open_shm maps a running rk512_server to a
server number and connects to it (see if it is alive).
The remaining functions now refer to the opened server by the server
number.

Naming conventions:
  The functions, which expect a server number as their first argument
have a 'm' just before the first '_' in their name ('m' stands for
multi-server).
All these functions do exist in a version missing this letter 'm'
and also missing the argument 'server'. These calls refer to the
standard server STDSVR and are implemented as MACROs.


*/

#include <sys/types.h>

#ifdef __cplusplus
extern "C" {
#endif


#ifndef STDSVR
#define STDSVR 0
#endif
// #define DEBUG 10

/* =================================================== */
/*  rk7_open_shm                                       */
/*                                                     */
/*   Initialisierung der  shared memory -Verbindung    */
/*   zu einem Server                                   */
/*                                                     */
/*   Initialization of the shared memory connection    */
/*   to a running rk512_server                         */
/* =================================================== */
/*

Funktion:
  Die Funktion analysiert die angegebene Konfigurationsdatei
und versucht, das dort durch den Eintrag SHMNAME spezifizierte
Shared Memory zu oeffnen. Bei Erfolg werden die zur Kommunikation
mit dem Server benoetigten Daten unter der Servernummer gespeichert.

  Falls in der Konfigurationsdatei der Eintrag SHMNAME auskommentiert
ist, so wird der Eintrag PORT zur Bestimmung des Shared Memory benutzt.


Eingangsparameter:
  int server      :  Servernummer, Bereich 0..9
  char * filename :  Name der Konfigurationsdatei

  long delaytime  :  max. Wartezeit des Servers vor Abmeldung

1.)
Der Parameter filename gibt den Namen der Konfigurationsdatei, des rc file s,
an.
Diese sollte bei Server und den Clients uebereinstimmen, also alle
verwenden die gleiche Konfigurationsdatei.

Die Clientsoftware wertet folgende Eintraege der Konfigurationsdatei aus:
        PORT            : Name der Schnittstelle, Voreinstellung fuer
                          SHMNAME
        SHMNAME           Shared Memory Name File, wird benoetigt zum
                          Verbindungsaufbau zum Server
        CLIENTDEBUG       Debuglevel fuer die Clients (Std. = 0)

Der Server wertet alle Eintraege ausser CLIENTDEBUG aus.


2.)
Der Parameter delaytime gibt an, nach welchem Zeitraum ein Client vom
Server fuer "tot" erklaert wird; erfolgt in dem durch delaytime spezi-
fizierten Zeitraum kein Zugriff des Clients auf seine Auftraege, so
gibt der Server diese wieder frei.
Dieser Mechanismus dient dazu, eine Blockade des Servers durch abgestuerzte
oder gekillte Clients (^C) zu verhindern.
Durch Angabe von -1 als delaytime wird diese Zeitueberwachung abgeschaltet.
Maximal zulässig: 2.000.000 Sekunden, das sind ca. 23 Tage.


  returncodes:
    0 = ohne Fehler
  -50 = ungueltige server-nummer bei rk512m_* - Aufruf
  -51 = rc-File nicht gefunden
  -52 = Fehler bei Auswertung des rc-files
  -53 = keine Verbindung zu shared memory moeglich


function:
  rk512m_open_shm analyzes the configuration file and tries to
open the Shared Memory which is specified by the SHMNAME entry.
If it succeeds, data necessary to communicate with the
server are stored and can be referred to from now on by
giving the server number.
  If the SHMNAME entry of the config file is masked by a comment sign '#',
this function will use the PORT entry to detect the shared memory.

------------------------

Input parameters:
  int server      :  server number, ranges from 0 up to 9
  char * filename :  name of the configuration file

  long delaytime  :  Client watchdog timer value

1.)
The parameter filename gives the name of the config file.
To avoid confusion, rk512_server and all it's clients should use
the same configuration file.

This function evaluates the following entries of the config file:
        PORT            : Name of the serial device, also used as
                          default for SHMNAME
        SHMNAME           Shared Memory Name File, used to
                          connect to the server
        CLIENTDEBUG       Debuglevel for the cient_library (Std. = 0)

(rk512_server does evaluate all entries except CLIENTDEBUG!)

2.)
The parameter delaytime denotes after which time this client will be
declared by the server to be dead; if the client does'nt access the server
within this time, the server will release the reserved memory.
This mechanism should avoid blocking servers memory by processes which
died (by error or signal).
Giving -1 here will turn off this. Maximal value: is 2000.000 seconds, 
about 23 days.


returncodes:
   0 = no errors
 -50 = invalid server number while using a rk512m (= multiserver) call
 -51 = rc-File not found
 -52 = error in rc-file
 -53 = cannot connect to the shared memory

*/

int rk7m_open_shm(int server,char* filename, long delaytime);

#define rk7_open_shm(f,d) rk7m_open_shm(STDSVR,f,d)


/* =================================================== */
/* rk7m_close_shm                                      */
/*                                                     */
/*     abmelden der rk*_server-Verbindung und          */
/*     Beenden aller Auftraege                         */
/*                                                     */
/*     close the rk*_server-connection and             */
/*     freeing all task blocks                         */
/* =================================================== */
/*

Funktion:
  Das mit rk7m_open_shm geoeffnete Shared Memory wird abgemeldet. 
  Falls noch Auftragsnummern geoeffnet sind, werden diese zuvor geschlossen. 
  Zu diesem Zweck wird fuer jede offene Auftragsnummer die Funktion
  rk7m_close_auftr aufgerufen.

Bemerkung:
  Falls beim Programm-Ende bereits alle Auftraege freigegeben sind, so
  eruebrigt sich dieser Aufruf, da das Betriebssystem neben dem Schliessen
  noch offener Dateien auch Shared Memories freigibt.

Eingangsparameter: 
  int server      :  Servernummer, Bereich 0..9
  int timeout     : Zeitlimit fuer die Kommunikation mit dem Server (Sekunden)
                  ( Die Wartezeit wird an die rk7_close_auftrt() -Aufrufe 
                    weitergereicht und gilt daher für jeden der noch offenen
                    Aufträge.)
  returncodes: 
    0 = ohne Fehler;
  -50 = ungueltige server-nummer bei rk7m_* - Aufruf
  -99 = timeout bei Kommunikation mit Server
 -100 = server shutdown



------------------------

function:
  Release the Shared Memory which was formerly opened by rk7m_open_shm.
  If there are reserved task blocks they will first be freed by
  calling rk7m_close_task

Remark:
  If at the end of the client application all task blocks are already
  freed, there is no need to call this function because the operation
  system will automatically free the shared memory.

Input parameters:
  int server    :  server number, range is 0..9
  int timeout   : limits the time availabe to communicate with the server (seconds)
                  This timeout is a parameter to all internally called rk*_close_auftr()
                  which are executed for each open task.
returncodes:
    0 = no errors;
  -50 = invalid server number while using a rk7m (= multiserver) call
  -99 = timeout reached in communicating with the server
 -100 = server shutdown



*/

int rk7m_close_shm(int server, int timeout);

#define rk7_close_shm(timeout) rk7m_close_shm(STDSVR,timeout)


/* =================================================== */
/* rk7m_open_task                                      */
/*                                                     */
/*    Initialisierung einer Auftragsverbindung         */
/*                                                     */
/*    Initializes a task block (returns a handle)      */
/* =================================================== */
/*

Funktion : reserviert beim Server einen Kommunikationsbereich,
           ueber den Auftraege abgewickelt werden.
           ( Ab 5.01 ist die Bufferlänge immer ausreichend.)


Eingangsparameter :
  int server      :  Servernummer, Bereich 0..9
  int timeout      : Zeitlimit fuer die Kommunikation mit dem Server (Sekunden)

     ergebnis >= 0  : Nummer des Auftragsbereichs 
                -20 : keine Nummer auf Clientseite mehr frei
                -21 : kein Bereich mehr frei
                -50 : ungueltige server-nummer bei rk512m_* - Aufruf
                -99 : timeout
               -100 : Server shutdown 


------------------------


function : reserve a task block at the server side,
           that means a communication area for tasks to connect to simatic

input parameters:
  int server       : server number, range is 0..9
  int timeout      : limits the time availabe to communicate with the server
                     (seconds)

returncodes: >=  0 : Number of the allocated task block (handle)
               -20 : all client sided numbers are allocated
               -21 : no range is free
               -50 : invalid server number while using a rk512m (= multiserver) call
               -99 : timeout server communication
              -100 : Server shutdown
*/

int rk7m_open_task(int server,int timeout);

#define rk7_open_task(t) rk7m_open_task(STDSVR,t)



/* =================================================== */
/* rk7m_close_task                                     */
/*                                                     */
/*     Abmeldung einer Auftragsverbindung              */
/*                                                     */
/*     Freeing an allocated task block                 */
/* =================================================== */
/*

Funktion: Der mit rk7m_open_task belegte Kommunikationsbereich 
          wird wieder freigegeben.

Eingangsparameter:
  int server      :  Servernummer, Bereich 0..9
  int a_nr        : Nummer des Auftrags
  int timeout     : Zeitlimit fuer die Kommunikation mit dem Server

Returncodes:
     0 = fertig ohne Fehler
   -16 = falsche Auftragsnummer  auf client-seite
   -99 = timeout bei Kommunikation
   -50 = ungueltige server-nummer bei rk512m_* - Aufruf
  -100 = Server shutdown

------------------------

function:
  release the communication area (server sided) which has formerly been allocated
  with rk7m_open_auftr.

input parameters:
  int server    : server number, range is 0..9
  int a_nr      : number of the task block (handle)
  int timeout   : limits the time availabe to communicate with the server

returncodes:
     0 = no errors
   -16 = the given task block number (handle) is wrong (client sided)
   -99 = timeout at server communication
   -50 = invalid server number while using a rk512m (= multiserver) call
  -100 = Server shutdown



*/

int rk7m_close_task(int server,int a_nr,int timeout);

#define rk7_close_task(a,t) rk7m_close_task(STDSVR,a,t)


/* =================================================== */
/* rk7m_fetch,                                         */
/* rk7m_fetch_prio,                                    */
/* rk7m_fetch_strt und                                 */
/* rk7m_fetch_prio_strt                                */
/*                                                     */
/*  Fetch-Auftrag an Server ( Daten von Simatic )      */
/*                                                     */
/*  Fetch-Order to the server ( get data from Simatic) */
/* =================================================== */
/*

!!
!! die rk7m* - Funktionen arbeiten Byte-orientiert, wogegen die
!! 'alten' rkm*-Funktionen Wort-orientiert arbeiten (S5-Kompatibilität)
!!
!! Der Koppelmerker-Mechanismus ist entfallen, er funktioniert nur bei
!! der RK512, die wird aber in Zusammenhang mit S7 fast nichtmehr eingesetzt.
!!

Funktion: Holen von Daten aus der Simatic.
Die uebergebenen Parameter werden zuerst auf ihre Zulaessigkeit geprueft.
Danach werden sie dem Server als FETCH-Auftrag uebergeben.

Hat der Server Zeit zur Durchfuehrung des Auftrages, so wird ein FETCH-
bzw. GET- Telegramm zur Simatic gesendet.
Die Antwort der Simatic wird im Auftragsblock abgelegt.
Folge-FETCH-Telegramme werden automatisch durchgeführt.

Die Funktionen rk7m_fetch und rk7m_fetch_prio warten auf die Beendigung
der Auftragsbearbeitung durch den Server, und uebertragen bei erfolgreicher
Durchfuehrung die Daten in den "buffer".

Die Funktionen rk7m_fetch_strt und rk7m_fetch_prio_strt stossen lediglich
den FETCH-Auftrag an. Die unten beschriebene Funktion rk7m_control dient
zum Abfragen des Bearbeitungsstandes, und fuehrt bei erfolgreichem Abschluss
den Transfer in den buffer durch.

PARAMETER:
    int server           :  Servernummer, Bereich 0..9
    unsigned char a_nr,     Nummer des Auftragsblocks
    unsigned char *buffer;  Buffer fuer die erwarteten Daten;
    char dbtyp;             Typ 'D' (immer 'D' )
    unsigned short dbnr;    Nr des Ziel-DBs
    int bytenr;             Nr 1. Byte in Ziel-DB
    int bytecnt;            Anzahl der zu holenden Bytes
    char cpu;               Ziel-CPU-Nummer  0 ..15
    int timeout;            in Sekunden
                              > 0 : Warten auf Quittierung von S5/MPX 
                              = 0 : Warten ohne Timeout-Begrenzung 
Rueckmeldungen:
    1  = fertig ohne Fehler (rk7m_fetch, ..fetch_prio )
    0  = fertig ohne Fehler (rk7m_fetch_*_strt)
   -1  = Fetch-Auftrag mit Fehler beendet (nur rk_fetch und rk_fetch_prio)
   -4  = dwtyp falsch
   -5  = dbnr falsch
   -6  = bytenr ausser Bereich
   -7  = bytecnt ausser Bereich
   -8  = bytenr + bytecnt ueberschreiten max. DB-Laenge 
   -10 = cpu-nr falsch
   -12 = falsche Auftragsnummer
   -13 = Auftragbuffer nicht mehr zugeteilt zu aktueller PID
   -14 = Auftragsnummer nicht geoeffnet
   -50 = ungueltige server-nummer bei rk7m_* - Aufruf
   -80 = keine Verbindung zu MPI-Adapter
   -99 = timeout bei Kommunikation
  -100 = server shutdown
  .. und andere ..


------------------------

!!
!! The rk7m* - functions are byte orientated, while the old rkm*- calls
!! are WORD oreientated (S5 compatibility)
!!
!! The rk7m* - functions do not support linking flags. because they are 
!! RK512 specific and RK512 is seldom used to link to a S7

function:
Fetch data from Simatic. These functions first check their parameters
if they are permissable. If there is no error, the server is told
to create a FETCH telegram to obtain the requested data from simatic.

Server sided information:
The server will convert the client's request to a FETCH or GET-telegram
which will be sent to the simatic. The response of the simatic will be
stored in the task block (referred to by the handle).

rk7m_fetch and rk7m_fetch_prio will wait for the fetch task to be
terminated (they block your application) and if the transfer was
successful they copy the data from server side to the client sided
"buffer".

rk7m_fetch_strt and rk7m_fetch_prio_strt will initiate data fetching
(they tell the server to do so) and then return without waiting
for termination of the transfer. To finally transfer the data
use the function rk7m_control (described below).

HINT:
Data transfer needs a task block which is large enough to contain
the requested data block.


input parameters:
    int server      :  server number, range is 0..9
    unsigned char a_nr,     number of the task block (handle)
    unsigned char *buffer;  Buffer for the requested data;
    char dbtyp;             Typ 'D' (always 'D' )
    unsigned short dbnr;    number of the source DB(DX)
    int bytenr;             number of 1.byte in source DB
    int bytecnt;            count of data bytes to read
    char cpu;               CPU-Number  0 .. 15
    int timeout;            in seconds
                              > 0 : Wait for response from simatic
                              = 0 : Wait without Timeout-limitation

returncodes:
    0  = no error
   -1  = Fetch-order terminated with error (only rk_fetch and rk_fetch_prio)
   -4  = dwtyp wrong
   -5  = dbnr wrong
   -6  = bytenr out of range
   -7  = bytecnt out of range
   -8  = bytenr + bytecnt is bigger than max. length of DB(DX)
  -10  = cpu-nr wrong
  -12  = task block number (handle) is wrong
  -13  = task block has been released by the server
  -14  = task block (handle) has not been opened
  -16  = task block (handle) on client side is wrong
  -50  = invalid server number while using a rk7m (= multiserver) call
  -99  = timeout at server communication
  -100 = server shutdown
   .. and more ..

*/

/* -------------- rk_fetch ---------------- */
extern int rk7m_fetch(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout);


#define rk7_fetch(anr,buf,dbt,dbn,dwn,dwa,cpu,t) \
rk7m_fetch(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t)


/* -------------- rk_fetch_prio ---------------- */
/* hochpriorer fetch-Aufruf */

extern int rk7m_fetch_prio(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout);


#define rk7_fetch_prio(anr,buf,dbt,dbn,dwn,dwa,cpu,t) \
rk7m_fetch_prio(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t)



/* -------------- rk_fetch_strt ---------------- */
extern int rk7m_fetch_strt(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout);


#define rk7_fetch_strt(anr,buf,dbt,dbn,dwn,dwa,cpu,t) \
rk7m_fetch_strt(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t)



/* -------------- rk_fetch_prio_strt ---------------- */
/* hochpriorer fetch-Aufruf */

extern int rk7m_fetch_prio_strt(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout);


#define rk7_fetch_prio_strt(anr,buf,dbt,dbn,dwn,dwa,cpu,t) \
rk7m_fetch_prio_strt(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t)




/* =================================================== */
/* rk7m_write,                                         */
/* rk7m_write_prio,                                    */
/* rk7m_write_strt und                                 */
/* rk7m_write_prio_str                                 */
/*                                                     */
/*  Write-Auftrag an Server ( Daten zu Simatic )       */
/*                                                     */
/*  Write-order to the server (send data to simatic)   */
/* =================================================== */
/*
!!
!!
!! die rk7m* - Funktionen arbeiten Byte-orientiert, wogegen die
!! 'alten' rkm*-Funktionen Wort-orientiert arbeiten (S5-Kompatibilität)
!!
!! Der Koppelmerker-Mechanismus ist entfallen, er funktioniert nur bei
!! der RK512, die wird aber in Zusammenhang mit S7 fast nichtmehr eingesetzt.
!!

Funktion:
Senden von Daten an die Simatic. 
Die uebergebenen Parameter und die Daten aus dem buffer werden in ein 
SEND- bzw.PUT-Telegramm zur Simatic umgesetzt und zum Partnergeraet
gesendet.

Die Funktionen rk7m_write und rk7m_write_prio warten auf die Beendigung
der Auftragsbearbeitung durch den Server, und melden anschliessend Erfolg
oder Misserfolg des Transfers.

Die Funktionen rk7m_write_strt und rk7m_write_prio_strt stossen lediglich
den  SEND-Auftrag an und uebergeben die Daten an den Server.

Wie schon beim fetch dient die unten beschriebene
Funktion rkm_control zum Abfragen des Bearbeitungsstandes.


PARAMETER:
    int server      :  Servernummer, Bereich 0..9
    unsigned char a_nr;     Auftragsnumer
    unsigned char *buffer;  Buffer mit Dateninhalten;
    char dbtyp;             Typ 'D' (immer 'D' )
    unsigned short dbnr;    Nr des Ziel-DBs
    int bytenr;             Nr 1. Byte in Ziel-DB
    int bytecnt;            Anzahl der zu holenden Bytes
    unsigned char cpu;      Ziel-CPU-Nummer  0 .. 3 ( oder ...15 mit MPX) 
    int timeout;            in Sekunden
                              > 0 : Warten auf Quittierung von S5/MPX 
                              = 0 : timeout aus, Warten auf Quittierung
                                    von Simatic/MPX
                              < 0 : Nur warten, bis Sende-Auftrag an RK512
                                    nicht auf Quittierung warten

Rueckmeldungen:
   2  = fertig ohne Fehler (r7m_write, _prio)
   0  = fertig ohne Fehler (rk7m_write_*_strt )
  -2  = Schreib-Auftrag mit Fehler beendet ( nur rk_write und rk_write_prio)
  -4  = dwtyp falsch
  -5  = dbnr falsch
  -6  = bytenr ausser Bereich
  -7  = bytecnt ausser Bereich
  -8  = bytenr + bytecnt ueberschreiten max. DB-Laenge 
  -10 = cpu-nr falsch
  -12 = falsche Auftragsnummer
  -13 = Auftragbuffer nicht mehr zugeteilt zu aktueller PID
  -14 = Auftragsnummer nicht geoeffnet
  -50 = ungueltige server-nummer bei rk7m_* - Aufruf
  -80 = keine Verbindung zu MPI-Adapter
  -99 = timeout bei Kommunikation mit dem Server
  -100= server shutdown
  .. und andere ..



------------------------
!!
!! The rk7m* - functions are byte orientated, while the old rkm*- calls
!! are WORD oreientated (S5 compatibility)
!!
!! The rk7m* - functions do not support linking flags. because they are 
!! RK512 specific and RK512 is seldom used to link to a S7

function:
Send data to Simatic. 
These functions first check their parameters if they are permissable.
If there is no error, the server is told to create a SEND or PUT telegram 
to transfer data to Simatic.

Server sided information:
The data to be sent will be copied to the task block on the server side.
The server will convert the client's request to an RK512-SEND-telegram
which will be sent to the simatic.

rk7m_write and rk7m_write_prio will wait for the SEND task to be
terminated (they block your application).

rk7m_write_strt and rk7m_write_prio_strt will initiate data sending
(they tell the server to do so) and then return without waiting
for termination of the transfer. To finally check for success of the
transfer use the function rk7m_control (described below).


input parameters:
    int server      :  server number, range is 0..9
    unsigned char a_nr      number of the task block (handle)
    unsigned char *buffer;  Buffer containig the data to be sent;
    char dbtyp;             Typ 'D' (always 'D' )
    unsigned short dbnr;    Number of the destination DB(DX)
    int bytenr;             number of 1.byte in destination DB
    int bytecnt;            count of data bytes to write
    char cpu;               CPU-Number  0 .. 15
    int timeout;            in seconds
                              > 0 : Wait for response from simatic
                              = 0 : Wait without Timeout-limitation

returncodes:
    0  = no error
   -2  = write-order terminated with error (only rk_write and rk_write_prio)
   -4  = dwtyp wrong
   -5  = dbnr wrong
   -6  = bytenr out of range
   -7  = bytecnt out of range
   -8  = bytenr + bytecnt is bigger than max. length of DB
  -10  = cpu-nr wrong
  -12  = task block number (handle) is wrong
  -13  = task block has been released by the server
  -14  = task block (handle) has not been opened
  -16  = task block (handle) on client side is wrong
  -50  = invalid server number while using a rk7m (= multiserver) call
  -80  = MPI-Adapter is not reachable ( using rk512_server)
  -99  = timeout at server communication
  -100 = server shutdown
   .. and more ..

*/
/* ----------- rk7_write -------------- */
extern int rk7m_write(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout );


#define rk7_write(anr,buf,dbt,dbn,dwn,dwa,cpu,t) \
rk7m_write(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t)



/* ----------- rk7_write_prio -------------- */

extern int r7km_write_prio(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout );


#define rk7_write_prio(anr,buf,dbt,dbn,dwn,dwa,cpu,t) \
rk7m_write_prio(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t)


/* ----------- rk7_write_strt -------------- */
extern int rk7m_write_strt(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout );


#define rk7_write_strt(anr,buf,dbt,dbn,dwn,dwa,cpu,t) \
rk7m_write_strt(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t)


/* ----------- rk7_write_prio_strt -------------- */

extern int rk7m_write_prio_strt(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout );


#define rk7_write_prio_strt(anr,buf,dbt,dbn,dwn,dwa,cpu,t) \
rk7m_write_prio_strt(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t)





/* ====================================================== */
/* rk7m_poll                                              */
/*                                                        */
/*  Poll-Auftrag an Server ( Daten zyklisch von Simatic ) */
/*                                                        */
/*  Poll-order to the server (to cyclically fetch data    */
/*                            from Simatic )              */
/* ======================================================
!!
!! die rk7m* - Funktionen arbeiten Byte-orientiert, wogegen die
!! 'alten' rkm*-Funktionen Wort-orientiert arbeiten (S5-Kompatibilität)
!!
!! Der Koppelmerker-Mechanismus ist entfallen, er funktioniert nur bei
!! der RK512, die wird aber in Zusammenhang mit S7 fast nichtmehr eingesetzt.
!!

Funktion:
Der Server wird beauftragt, zyklisch Daten von der Simatic zu holen.
Nach Erteilung der poll-Auftrags fuehrt der Server im angegebenen Zeitraster
FETCH-Auftraege durch.

Die Funktion rkm_control dient wie bei rk7m_fetch_strt und rkm_fetch_prio_strt
zur Statusabfrage sowie zum Datentransfer vom Server zum Client.


PARAMETER:
    int server      :  Servernummer, Bereich 0..9
    unsigned char a_nr;     Auftragsnumer
    unsigned char *buffer;  Buffer mit Dateninhalten;
    char dbtyp;             Typ 'D' (immer 'D' )
    unsigned short dbnr;    Nr des Ziel-DBs
    int bytenr;             Nr 1. Byte in Ziel-DB
    int bytecnt;            Anzahl der zu holenden Bytes
    unsigned char cpu;      Ziel-CPU-Nummer  0 .. 3 ( oder ...15 mit MPX) 
    int timeout;            in Sekunden
                              > 0 : Warten auf Quittierung von S5/MPX 
                              = 0 : timeout aus, Warten auf Quittierung 
                                    von S5/MPX
                              < 0 : Nur warten, bis Sende-Auftrag an RK512
                                    nicht auf Quittierung warten

    long polltime;          Pollzeit in Millisekunden


Rueckmeldungen:
    0 = fertig ohne Fehler
   -4 = dwtyp falsch
   -5 = dbnr falsch
   -6 = dwnr ausser Bereich
   -7 = dwanz ausser Bereich
   -8 = dwnr + dwanz ueberschreiten max. DB-Laenge 
  -10 = cpu-nr falsch
  -12 = falsche Auftragsnummer
  -13 = Auftragbuffer nicht mehr zugeteilt zu aktueller PID
  -14 = Auftragsnummer nicht geoeffnet
  -19 = Pollzeit ausserhalb Bereich
  -50 = ungueltige server-nummer bei rk7m_* - Aufruf
  -80 = keine Verbindung zu MPI-Adapter
  -99 = timeout bei Kommunikation
  -100= server shutdown



------------------------
!!
!! The rk7m* - functions are byte orientated, while the old rkm*- calls
!! are WORD oreientated (S5 compatibility)
!!
!! The rk7m* - functions do not support linking flags. because they are 
!! RK512 specific and RK512 is seldom used to link to a S7

function:
Instruct the server to repeat fetches at given time intervals.
Data requests will be sent to the simatic and results are stored
in the task buffer (server sided). Use rk7m_control to get the data
into the client applications data area.


input parameters:
    int server      :  server number, range is 0..9
    unsigned char a_nr,     number of the task block (handle)
    unsigned char *buffer;  Buffer for requested data;
    char dbtyp;             Typ 'D' (always 'D' )
    unsigned short dbnr;    Number of the destination DB(DX)
    int bytenr;             number of 1.byte in source DB
    int bytecnt;            count of data bytes to read
    char cpu;               CPU-Number  0 .. 3
    int timeout;            in seconds
                              > 0 : Wait for response from simatic
                              = 0 : Wait without Timeout-limitation

    long polltime;          time interval between two fetches (milliseconds)

output parameters:
    unsigned char *auftrstat; status byte of the 3964R
    unsigned char *stat512;   status byte of the RK512

returncodes:
    0  = no error
   -4  = dwtyp wrong
   -5  = dbnr wrong
   -6  = dwnr out of range
   -7  = dwanz out of range
   -8  = dwnr + dwanz is greater than max. length of DB(DX)
  -10  = cpu-nr wrong
  -12  = task block number (handle) is wrong
  -13  = task block has been released by the server
  -14  = task block (handle) has not been opened
  -16  = task block (handle) on client side is wrong
  -19  = polltime out of range
  -50  = invalid server number while using a rk7m (= multiserver) call
  -80  = MPI-Adapter is not reachable ( using rk512_server)
  -99  = timeout at server communication
  -100 = server shutdown

*/

/* ----------- rk7_poll -------------- */

extern int rk7m_poll(
    int server,
    unsigned char a_nr,
    unsigned char *buffer,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int bytecnt,
    unsigned char cpu,
    int timeout,
    long polltime);


#define rk7_poll(anr,buf,dbt,dbn,dwn,dwa,cpu,t,pt) \
rk7m_poll(STDSVR,anr,buf,dbt,dbn,dwn,dwa,cpu,t,pt)


/* =================================================== */
/* rk7m_set_time_strt,                                 */
/*                                                     */
/*  aktuelle Zeit zu Simatic schreiben                 */
/*  (für Kopplungen, bei denen das NTP-Verfahren       */
/*   noch nicht geht )                                 */
/*                                                     */
/*  Write-order to the server (send data to simatic)   */
/* =================================================== */
/*
!!
Funktion:
Senden der aktuellen Uhrzeit an die Simatic.
Es gibt nur die _strt-Funktion, da der Erfolg nicht abgewartet wird. 

rk_control und rk_control2 arbeiten wie bei einem rk_write*_strt()-Aufruf.

Die Nummer der CPU (siehe Beschreibung des jeweiligen Servers),
die Nummer des Datenbausteins und das Startbyte sind einstellbar.

Es ist zusätzlich eine Uhrzeitverschiebung einstellbar,um die Zeit an die
in der S7-CPU-Uhr benutzte Zeitzone einzustellen.
Dabei gilt: 
  adjust  = 0 : es wird UTC übertragen
          = 1 : Lokalzeit ohne Sommerzeit-Korrektur
          = 2 : Lokalzeit inclusive Sommerzeit-Korrektur
         sonst: [Sek] Zeitverschiebung gegenüber UTC ohne Sommerzeitkorrektur.
                Beispiel : Duesseldorf ist 1 Stunde vor UTC, also 3600



Die Struktur des Datenblocks, der zu der S7 geschrieben wird, ist fest 
definiert:

setze_zeit Struct
  zeit : DATE_AND_TIME;  //Uhrzeit
  trigger : INT;         //Falls Trigger, dann 1, wird von CPU rückgesetzt.
END_STRUCT;


PARAMETER:
    int server      :  Servernummer, Bereich 0..9
    unsigned char a_nr;     Auftragsnumer
    char dbtyp;             Typ 'D' (immer 'D' )
    unsigned short dbnr;    Nr des Ziel-DBs
    int bytenr;             Nr 1. Byte in Ziel-DB
    int adjust;             Korrektur-Wert (siehe Beschreibung)
    unsigned char cpu;      Ziel-CPU-Nummer  0 .. 3 ( oder ...15 mit MPX) 
    int timeout;            in Sekunden
                              > 0 : Warten auf Quittierung von Server
                              = 0 : timeout aus, Warten auf Quittierung 
                                    von S5/MPX
                              < 0 : Nur warten, bis Sende-Auftrag an RKtcp_server
                                    nicht auf Quittierung warten

Rueckmeldungen:
   0  = fertig ohne Fehler
  -4  = dwtyp falsch
  -5  = dbnr falsch
  -6  = bytenr ausser Bereich
  -10 = cpu-nr falsch
  -12 = falsche Auftragsnummer
  -13 = Auftragbuffer nicht mehr zugeteilt zu aktueller PID
  -14 = Auftragsnummer nicht geoeffnet
  -16  falsche Auftragsnummer auf client-Seite
  -50 = ungueltige server-nummer bei rk7m_* - Aufruf
  -80 = keine Verbindung zu MPI-Adapter
  -99 = timeout bei Kommunikation mit dem Server
  -100= server shutdown
  .. und andere ..



------------------------
Function:
Send the actual date and time  to a Simatic.
Only  _strt-function exists, no wait for feedback.

rk_control and rk_control2 act like on a rk_write*_strt()-call.

The number of the CPU, the number of Datablock and first data byte
has to be defined. .

an additional time adjust value exists,to define the deviation according 
to UTC
That is
  adjust  = 0 : transfer of UTC
          = 1 : local time without correction of daylight saving time (DST)
          = 2 : local time including correction of daylight saving time (DST)
         else : [sec] time deviation relative to UTC, not including DST correction
                i.e.:  Duesseldorf is 1 hour earlier than UTC, meens 3600



The structure to be sent is a fixed definition:

set_time STRUCT
  zeit : DATE_AND_TIME;  // date and time
  trigger : INT;         //will be set by server, (= new.DAT) needs to be reset by plc
END_STRUCT;

input parameters:
    int server      :  server number, range is 0..9
    unsigned char a_nr      number of the task block (handle)
    char dbtyp;             Typ 'D' (always 'D' )
    unsigned short dbnr;    Number of the destination DB(DX)
    int bytenr;             number of 1.byte in destination DB
    int adjust;             adjustment value
    char cpu;               CPU-Number  0 .. 15
    int timeout;            in seconds
                              > 0 : Wait for response from server
                              = 0 : Wait without Timeout-limitation


returncodes:
    0  = no error
   -4  = dwtyp wrong
   -5  = dbnr wrong
   -6  = bytenr out of range
  -10  = cpu-nr wrong
  -12  = task block number (handle) is wrong
  -13  = task block has been released by the server
  -14  = task block (handle) has not been opened
  -16  = task block (handle) on client side is wrong
  -50  = invalid server number while using a rk7m (= multiserver) call
  -80  = MPI-Adapter is not reachable ( using rk512_server)
  -99  = timeout at server communication
  -100 = server shutdown
   .. and more ..

*/
/* ----------- rk7m_set_time_strt -------------- */
extern int rk7m_set_time_strt(
    int server,
    unsigned char a_nr,
    char dbtyp,
    unsigned short dbnr,
    int bytenr,
    int adjust,
    unsigned char cpu,
    int timeout );


#define rk7_set_time_strt(anr,dbt,dbn,dwn,adj,cpu,t) \
rk7m_set_time_strt(STDSVR,anr,dbt,dbn,dwn,adj,cpu,t)



/* ============================================================ */
/*  rk7m_control                                                */
/*  Abarbeitung der Datentransfers und Status)                  */
/*  state check and data transfer                               */
/* ============================================================

rk7m_control entspricht der alten rk_control2, d.h.
die positiven Rueckmeldungen 1-3 erscheinen nur fuer die Dauer eines Aufrufs 
ab den nächsten Aufrufen folgt 4 (idle) oder, falls zwischenzeitlich ein neuer 
Auftrag gestartet wurde, 0

Die function prueft den Zustand eines Auftrags und holt gegebenenfalls Daten.

Wurden mit einem rk7*_fetch*_start() oder rk7m_poll() Daten angefordert:

falls returncode 
= 0 : zuvor angstoßener Auftrag läuft noch.

= 1: Daten wurden empfangen und innerhalb dieses rk7m_control() - Aufrufs 
     übergeben.
     ( Die notwendige Zieladressse wurde durch den anfordernden Aufruf
     rk7m_fetch*_start() übergeben.)

= 2: Daten wurden erfolgreich gesendet.
     Dies ist die Anwort auf einen zuvor erteilten Auftrag rk7m_write_*_start()

= 3: Daten des Poll-Auftrags wurden empfangen und innerhalb dieses
     rk7m_control() - Aufrufs übergeben.
     ( Die notwendige Zieladressse wurde durch den anfordernden Aufruf
     rk7m_poll() übergeben.)

= 4: idle, bereit für neuen Auftrag

< 0: fehlermeldung

!! rk_control() and rk_control2() unterstützen eine Wartezeit auf 
!! Milli-Sekunden-Basis.
!! Diese Zusatzfunktionalität ist nicht mehr notwendig:
!!
!! Als rk_control() und rk_control2() definiert wurden, war Linux noch ziemlich
!! jung und hat offiziell ( man - pages ) noch keinen sleep kleiner als
!! eine Sekunde unterstützt. Das War im Jahr 1994 !!
!!
!! Seit langen Jahren gibt es aber bereits usleep().
!! Statt der früheren Funktionalität mit "a_nr = 100 sollte jetzt usleep()
!! benutzt werden.
!!


PARAMETER:
    int server                  Servernummer, Bereich 0..9
    unsigned char a_nr          Auftragsnummer
    unsigned char *funktion     gerade bearbeitete Funktion
                                0 kein Auftrag
                                1 fetch
                                2 write
                                3 poll
                                Diese Rückmeldung ist nur für Diagnose-Zwecke

Rueckmeldungen:
   4  = Auftrag ist "idle" (frei fuer neuen Auftrag)
   3  = Poll - Daten ohne Fehler gelesen 
   2  = Write ohne Fehler beendet
   1  = Fetch ohne Fehler beendet
   0  = Auftrag laeuft noch
  -1  = Fetch mit Fehler beendet	(*)
  -2  = Write mit Fehler beendet	(*)
  -3  = Poll-Daten fehlerhaft gelesen	(*)
  -4  = dwtyp falsch
  -5  = dbnr falsch
  -6  = bytenr ausser Bereich
  -7  = bytecnt ausser Bereich
  -8  = bytenr + bytecnt ueberschreiten max. DB-Laenge 
  -10  = cpu-nr falsch
  -12  = falsche Auftragsnummer
  -13  = Auftragbuffer nicht mehr zugeteilt zu aktueller PID
  -14  = Auftragsnummer nicht geoeffnet 
  -15  = Auftrag wurde fuer write angestossen, Fetch nicht aktiv
  -16  = falsche Auftragsnummer auf client-Seite
  -17  = falsche Schrittnummer: Bearbeitung asynchron
  -18  = ungueltige Auftragsart ( kein Fetch/Write/Poll)";
  -19  = Pollzeit ausserhalb zulaessigem Bereich";
  -20  = alle client-seitigen Auftragsnummern belegt";
  -21  = Anmeldung: keine Auftragsnummer frei";
  -31  = IP-Kopplung: Ziel-AG nicht definiert
  -32  = IP-Kopplung: keine Verbindung zum AG
  -35  = IP-Kopplung: Timeout beim Senden
  -36  = IP-Kopplung: Timeout beim Empfang Reaktionstelegramm
  -37  = IP-Kopplung: Reaktionstelegramm Header falsch (assynchron)
  -38  = IP-Kopplung: Fehlermeldung vom AG
  -39  = IP-Kopplung: Angeforderte Daten nicht komplett (RFC1006-Fehler)
  -40  = IP-Kopplung: Timeout beim Empfang Daten
  -41  = IP-Kopplung: RFC1006-Fehler bei CONNECTION REQUEST
  -42  = IP-Kopplung: S7-Kommunikationsfehler bei CONNECTION REQUEST
  -43  = IP-Kopplung: bei CONNECTION REQUEST zugeteilter S7-Kommunikationsbereich zu klein 
  -44  = IP-Kopplung: unspezifizierter Fehler in S7-Kommunikation
  -50  = ungueltige server-nummer bei rk7m_* - Aufruf
  -70  = RESTRICTED_ACCESS: kein Leserecht fuer diesen DB
  -71  = RESTRICTED_ACCESS: kein Schreibrecht fuer diesen DB
  -80  = MPI-Adapter ist nicht erreichbar ( using rk512_server)
  -98  = keine shm-Verbindung zum rk*-server
  -99  = timeout bei Kommunikation
  -100 = server shutdown

------------------------

rk7m_control conforms to the old rk_control2()


This function checks for the status of a formerly given order and
fetches data (from the server) if they are available.

The positive return codes 1-3 are only returned by one call, the next call 
will return 4 ( idle) or 0 ( order still in work) if a new order
was started.

case returncode
= 0 : formerly started task is still running.

= 1: Data have been received and written to the destination buffer while
     executing this call.
     ( The necessary destination address has been declared by the formarly 
      processed rk7m_fetch*_start() .)

= 2: Data have been sent sucsessfully.
     This is the result of a formaerly processed rk7m_write_*_start().

= 3: Data have been received and written to the destination buffer while
     executing this call.
    ( The necessary destination address has been declared by the formarly
      processed rk7m_poll(). )

= 4: idle, ready for a new job

< 0: error message

!! rk_control() and rk_control2() support a milli seconds based sleep.
!! This extra functionality enabled by "a_nr = 100" is obsolete.
!!
!! When rk_control() and rk_control2() were created, Linux was quite young and
!! did not support a sleep based on milli seconds. This was in the year 1994 !!
!!
!! Since long years, usleep() is supported, we recommend to use this call instead.
!!

PARAMETERS

input parameters:
    int server          :  server number, range is 0..9
    unsigned char a_nr  :  number of the task block (handle);

output parameters:
    unsigned char *funktion,	code of the currently performend
                                function. codes are:
                                0 no job
                                1 fetch
                                2 write
                                3 poll
                                This feedback is only useful for diagnostic 
                                reasons.

returncodes:
    4  = Idle - no actual tast to process
    3  = Poll - Data read without error
    2  = Write terminated without error
    1  = Fetch terminated without error
    0  = order is still in work
   -1  = Fetch terminated with error	(*)
   -2  = Write terminated with error	(*)
   -3  = Poll-Data have errors	(*)
   -4  = dwtyp wrong
   -5  = dbnr wrong
   -6  = bytenr out of range
   -7  = bytecnt out of range
   -8  = bytenr + bytecnt is bigger than max.length of DB(DX)
  -10  = cpu-nr wrong
  -12  = task block number (handle) is wrong
  -13  = task block has been released by the server
  -14  = task block (handle) has not been opened
  -16  = task block (handle) on client side is wrong
  -17  = invalid Stepnumber: processing is asynchron
  -18  = invalid order code ( no Fetch/Write/Poll)
  -19  = polltime out of range
  -20  = all client sided numbers are allocated ( 0..9 )
  -21  = no more free handles available"
  -22  = requested data range is too big, or all data buffers are allocated
  -31  = IP-link: destination plc not defined
  -32  = IP-link: no connection to plc
  -35  = IP-link: Timeout while writing to the plc
  -36  = IP-link: Timeout while receiving the REATEL reaction telegram
  -37  = IP-link: wrong header in the reaction telegram: asynchronous
  -38  = IP-link: error message form plc
  -39  = IP-link  requested data not complete (internal error of RFC1006)
  -40  = Timeout while receiving data from the plc
  -41  = IP-link error on RFC1006 while CONNECTION REQUEST
  -42  = IP-link error on S7-communication while CONNECTION REQUEST
  -43  = IP-link error: S7-communication block too small
  -44  = IP-link error: undefined S7com error
  -50  = invalid server number while using a rk512m (= multiserver) call
  -70  = RESTRICTED_ACCESS: no permission to read from this DB
  -71  = RESTRICTED_ACCESS: no permission to write to this DB

  -80  = MPI-Adapter is not reachable ( using rk512_server)
  -98  = no connection to shm to rk*-server
  -99  = timeout at server communication
  -100 = server shutdown
*/

extern int rk7m_control(
    int server,
    unsigned char a_nr,
    unsigned char *funktion );

#define rk7_control(anr,fkt)  \
rk7m_control(STDSVR,anr,fkt)



/* ====================================================== */
/* clinit_result (Diagnosehilfe bei der Initialisierung)  */
/*                                                        */
/* clinit_result (Diagnostic aid upon initialization time)*/
/* ======================================================
Funktion: liefert einen zu "nr" passenden Fehlertext

Beim Aufruf der rk7* -Funktionen wird  clinit_result() nich mehr benötigt,
benutzen Sie statt dessen rk7_errortext ();


------------------------

On using rk7* - functions, clinit_result() is obsolete.
Use rk7_errortext_en () instead.

*/

/* ================================================*/
/* rk7_errortext                                   */
/*                                                 */
/*  generiert aus Fehlernummern einen Fehlertext   */
/*                                                 */
/*  generates an error message from error numbers  */
/* ================================================*/
/*
    verwendbar für alle rk7* - Aufrufe

------------------------
    usable for all rk7* - calls

*/


/* ================================================*/

/* Deutsche Version / german version */
char * rk7_errortext (int fcode);


/*
  Die Fehlertexte sind

     4  idle, bereit fuer neuen Auftrag
     3  Poll-Daten ohne Fehler gelesen
     2  Write ohne Fehler beendet
     1  Fetch ohne Fehler beendet
     0  Auftrag laeuft noch
    -1  Fetch mit Fehler beendet (*)
    -2  Write mit Fehler beendet (*)
    -3  Poll-Daten fehlerhaft    (*)
    -4  dwtyp falsch
    -5  dbnr falsch
    -6  Start-Byte-Nr ausser Bereich
    -7  Byte-Anzahl ausser Bereich
    -8  Start-Byte + Anzahl ueberschreiten max. DB-Laenge
    -9  kmbit falsch
   -10  cpu-nr falsch
   -11  Datenbereich auf server zu klein
   -12  falsche Auftragsnummer
   -13  Auftragbuffer nicht mehr zugeteilt zu aktueller PID
   -14  Auftragsnummer nicht geoeffnet
   -15  Auftrag wurde fuer write angestossen, Fetch nicht aktiv
   -16  falsche Auftragsnummer auf client-Seite
   -17  falsche Schrittnummer: Bearbeitung asynchron
   -18  ungueltige Auftragsart ( kein Fetch/Write/Poll)
   -19  Pollzeit ausserhalb zulaessigem Bereich
   -20  alle client-seitigen Autragsnummern belegt
   -21  Anmeldung: keine Auftragsnummmer frei
   -22  Anmeldung: kein Datenbereich der gewuenschten Groesse verfuegbar
   -23  Datenbaustein existiert nicht oder nicht zugreifbar
   -24  ungeradzahlige Byte-Adressen nur bei S7-Kommunikation erlaubt
   -25  ungeradzahlige Byte-Laengen nur bei S7-Kommunikation erlaubt
   -26  Auftrag bereits aktiv
   -31  IP-Kopplung: Ziel-AG nicht definiert
   -32  IP-Kopplung: keine Verbindung zum AG
   -35  IP-Kopplung: Timeout beim Senden
   -36  IP-Kopplung: Timeout beim Empfang Reaktionstelegramm
   -37  IP-Kopplung: Reaktionstelegramm Header falsch (assynchron)
   -38  IP-Kopplung: Fehlermeldung vom AG
   -39  IP-Kopplung: Angeforderte Daten nicht komplett (RFC1006-Fehler)
   -40  IP-Kopplung: Timeout beim Empfang Daten
   -41  IP-Kopplung: RFC1006-Fehler bei CONNECTION REQUEST
   -42  IP-Kopplung: S7-Kommunikationsfehler bei CONNECTION REQUEST
   -43  IP-Kopplung: zugeteilter S7-Kommunikationsbereich zu klein
   -44  IP-Kopplung: unspezifizierter Fehler in S7-Kommunikation
   -45  IP-Kopplung: Formatfehler in RFC1006-PDU Reaktionstelegramm
   -50  Ungueltige server-Nummer in einem rk512m-..Aufruf
   -51  rk7_open_shm(): rc-File nicht gefunden
   -52  rk7_open_shm(): Fehler bei Auswertung des rc-files
   -53  rk7_open_shm(): keine Verbindung zu shared memory moeglich

   -55  TTY BREAK erkannt
   -56  TTY Zeichenfehler(PARITY/FRAME)
   -57  TTY Zeichenfehler(unspezifiziert)

   -60  Protokollfehler in Kommunikationsschicht seriell
   -61  Protokollkopffehler in Kommunikationsschicht seriell
   -62  Potokollfehler: illegales Komando in Kommunikationsschicht seriell
   -63  CPU in Stop: Meldung von Kommunikationsschicht seriell
   -64  Ziel-Baustein blockiert durch Koppelmerker
   -65  Protokollfehler in Folgetelegramm Kommunikationsschicht seriell
   -66  Timeout Empfang Kommunikationsschicht seriell
   -67  Timeout Senden Kommunikationsschicht seriell
   -70  RESTRICTED_ACCESS: kein Leserecht fuer diesen DB
   -71  RESTRICTED_ACCESS: kein Schreibrecht fuer diesen DB

   -80  MPI-RK512-Adapter nicht erreichbar

   -81  3964: Buffer-Überlauf Empfang
   -82  3964: BCC-Error Empfang
   -83  3964: Empfangsbuffer belegt,Telegramverlust ??
   -84  3964: Initialisierungskonflikt
   -85  3964: Timeout ZVZ beim Empfang
   -86  3964: Timeout beim BCC-Empfang

   -98  keine shm-Verbindung zum rk*-server
   -99  timeout bei Client-Server-Kommunikation
   -100 server shutdown


*/

/* Englische Version / english version */
char * rk7_errortext_en (int fcode);

/*
the eror textes are
     4  task idle ( ready to be used)
     3  Poll - Data read without error
     2  Write terminated without error
     1  Fetch terminated without error
     0  order is still in work
    -1  Fetch terminated with error (*)
    -2  Write terminated with error (*)
    -3  Poll-Data have errors    (*)
    -4  dwtyp wrong
    -5  dbnr wrong
    -6  start byte number out of range
    -7  count of bytes out of range
    -8  start byte + count exeed max. length of DB(DX)
    -9  kmbit wrong
   -10  cpu-nr wrong
   -11  task block (server sided) is too small for this request
   -12  task block number (handle) is wrong
   -13  task block has been released by the server
   -14  task block (handle) has not been opened
   -15  task was started for a write task, fetch is inactive
   -16  task block (handle) on client side is wrong
   -17  invalid step number: processing is asynchronous
   -18  invalid order code ( no Fetch/Write/Poll)
   -19  polltime out of range
   -20  all client sided numbers are allocated ( 0..9 )
   -21  no more free handles available
   -22  requested data range is too big, or all data buffers are allocated
   -23  requested data block not existing or not accessible
   -24  odd byte addresses only allowed by s7 communication
   -25  odd byte length only allowed by s7 communication
   -26  job already running
   -31  IP-link: destination plc not defined
   -32  IP-link: no connection to plc
   -35  IP-link: Timeout while writing to the plc
   -36  IP-link: Timeout while receiving the REATEL reaction telegram
   -37  IP-link: wrong header in the reaction telegram: asynchronous
   -38  IP-link: error message form plc
   -39  IP-link  requested data not complete (internal error of RFC1006)
   -40  Timeout while receiving data from the plc
   -41  IP-link error on RFC1006 while CONNECTION REQUEST
   -42  IP-link error on S7-communication while CONNECTION REQUEST
   -43  IP-link error: S7-communication block too small
   -44  IP-link error: undefined S7com error 
   -45  IP-link format error in RFC1006-PDU indication telegram
   -50  invalid server number while using a rk512m (= multiserver) call
   -51  rk7_open_shm(): rc-File not found
   -52  rk7_open_shm(): error in rc-file
   -53  rk7_open_shm(): cannot connect to the shared memory

   -55  TTY BREAK detected
   -56  TTY PARITY or FRAME error
   -57  TTY error (unspecitied)

   -60  protocol error in serial communication
   -61  protocol error in header on serial communication
   -62  protocol error: illegal command in serial communication
   -63  CPU in Stop: message of serial communication
   -64  destination data block blocked by K-flag
   -65  Protocol error in follow up tel. of serial communication
   -66  Timeout on receiving in serial communication
   -67  Timeout on sending in serial communication
   -70  RESTRICTED_ACCESS: no permission to read from this DB
   -71  RESTRICTED_ACCESS: no permission to write to this DB

   -80  MPI-Adapter is not reachable ( rk512_server and option MPI2RK ON)

   -81  3964: buffer-overflow on receive
   -82  3964: BCC-Error on receive
   -83  3964: receive buffer occupied. loss of telegram ??
   -84  3964: Initialisation conflict
   -85  3964: timeout ZVZ on receiving
   -86  3964: timeout on receiving BCC

   -98  no connection to shm to rk*-server
   -99  timeout at client-server communication
   -100 server shutdown


*/

/* =================================================== */
/*     Ausgabe der Versionsnummer der                  */
/*     rk7_client -Bibliothek als Textstring           */
/*                                                     */
/*     print the version number of rk7_client.o        */
/*            as a text string                         */
/* =================================================== */

char *rk7_version(char s[]);


/* =================================================== */
/*  rk7m_svr_info                                      */
/*                                                     */
/*   Ausgabe von Infos zum Server                      */
/*                                                     */
/*   output of infos of the attached server            */
/*   to a running rk512_server                         */
/* =================================================== */
/*
( since Version 4.13 !! )

Funktion:
Die Funktion bringt Informatioenn über den angesprocnenen Server zurueck.

Eine structure vom Typ "rk_svr_inf_type" muß an die function übergeben werden.
Sie wird vom Aufruf ausgefüllt.
Format: siehe "rk512_client.h"

Parameter:
  int server      :  Servernummer, Bereich 0.. 9
  int *unlim      :  Rueckgabe: 
                      1 = Server kann unbegrenzt laufen
                      0 = Laufzeit des Servers ist begrenzt
  rk_svr_inf_type * infblk; Structure of type of rk_svr_inf_type


Rueckmeldungen:
   0  = ohne Fehler beendet
 -50  = Ungueltige server-Nummer in einem rk512m-..Aufruf
 -98  = keine shm-Verbindung zum rk*-server

------------------------


This function returns information of the attachedd server.

A structure of type "rk_svr_inf_type" is needed to return the inforations.
Format: see "rk512_client.h"

input parameters:
  int server      :  Servernummer, Bereich 0.. 9
  int *unlim      :  on return: 
                      1 = server run time unlimited
                      0 = server run time limited
  rk_svr_inf_type * infblk; Structure of type of rk_svr_inf_type


output parameters:
   tHe filled structure 

returncodes:
   0  = terminated without error
 -50  = invalid server number while using a rk512m (= multiserver) call
 -98  = no connection to shm to rk*-server

*/

#ifndef RK_SVR_INFTYPE
#define RK_SVR_INFTYPE

/* zusatzinformationen über tcpserver: */
/* erweitert */
typedef struct tcp_info_t {  /* bei 16 : 56 Bytes */
	int64_t	last_connect; /* Zeitpunkt letzter kompletter Connect (8) */
	int64_t	last_unconnect; /* Zeitpunkt letzter unConnect, mindestens 1 Zweig (16) */
	int32_t write_cnt; /* Anzahl Schreibvorgänge    (20) */
	int32_t fetch_cnt; /* Anzahl fetchvorgänge      (24) */
	int16_t wstate; /* Status schreibtask     (26)  */
	int16_t fstate; /* Status fetchtask       (28)  */
	signed char com_type;	/* Kommunikationstyp -1 = nicht definiert
			   0 = TCP-Raw, 1 = Iso_on_tcp, 2 = s7-kommunikation (29) */
	signed char mpx_idx;	/* Index für Multiplex-Bereich (30) */
	signed char mpx_idx_ack;/* Index für Multiplex-Bereich  Antwort an Client (31) */
	signed char mpx[25];
} tcp_info_type;



/* ------ Feststellung des Zielbetriebsystems 32Bit /64 Bit des servers --- */

/* Zusatzinformationen über Client und Server 07.10.2006 
   Der Bereich muß 1024 Byte gross sein
*/
typedef struct rk_svr_inf_type {      /* Wird von Server beschrieben */
  char svrtyp[16];        /* z.bB: 'rk512_server' */
  char version[16];       /* Serverversion Wird von Server beschrieben */
  char compile_date[16];  /* Kopilierdatum des aktuellen servers */
  char runmode[32];       /* Infos ueber die Laufzeit des Servers:
                             z.B: 'LICENSE' Siehe Lizenz-Zustände  in rk51x_key.h */
  char  stop_zeit[32] ;   /* Zeit wenn der server stoppt, wenn runmode != unbegrenzt */
  unsigned char osbits ; /* Version des Servers für OS Bits z.B 32,64 */
  unsigned char svrAuftrAnz ; /* Maximale Anzahl der Auftraege fuer Server */
  unsigned char ag_anz;	  /* Anzahl der bearbeitbaren AGs dieser Server    */
  unsigned char res_114[5];
  union {
    struct {
	unsigned char res[904]; /* Reserve, auffuellen bis 1024 */
    } leer; /* Reserve, auffuellen bis 1024 */
    tcp_info_type tcp[1] ; /* bei Die Anzahl ist in 'ag_anz' */
  } ;
} Rk_svr_inf_type;
#endif

int rk7m_svr_info (int server,int *unlim, Rk_svr_inf_type *infblk) ;

#define rk7_svr_info(f,g) rk7m_svr_info (STDSVR,f,g)


#ifdef __cplusplus
}
#endif
