/* ====================================================================
   Rechnerkopplung zu SIMATIC-S5/S7  mit Prozedur RK512 / RK511
     am Beispiel von Pult, einer in xforms geschriebenen Simatic-Bedienoberflaeche

File: simatic_link.c

 (C)  Heisch Automation
      Im Vorderen Grosstal 4
      76857 Albersweiler
 und
      Gerald Emig
      Auf der Steige 7
      66564 Ottweiler/Fuerth


   =====================================================================
   Pult 2 Version 0.1
   19.02.2008  HW   simatic_link.h eingefuegt, Variablen fuer Anwenderprogramm 
                    verfuegbar gemacht
   Stand 0.2
   17.07.2000 HW fetch_ready() gibt im Fehlerfall Fehlerkennung aus, 
              statt idle ( vorheriger work around.
   20.06.2000 Fehleranzeige, wenn server nicht laeuft 
   06.05.2000

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>

#include "simatic_link.h"

/* =================================================== */
/* Datenbereich lokal */
                                /* Auftragsnummern, signed, damit Fehler */
static signed char auftr_nr_f = -100 ;        /* Auftragsnummer des Lese-Auftrags */
static signed char auftr_nr_wr = -100;        /* Auftragsnummer des Schreib-Auftrags */


/* Datenbereich */
/* Zuerst werden die Speicherplaetze fuer die Auftragsnummern definiert.
   Grundsaetzlich koennen die Auftragsnummern abwechslend fuer
   Schreib- und Leseauftraege benutzt werden. Hauptsache ist, dass
   die Datenpakete in den Datenbereich, der im Server der Auftragsnummer
   zugeordnet ist, auch alle hineinpassen.
   Normalerweise wuerde fuer sequentielle Write- und Fetch-Auftraege somit
   eine einzige Auftragsnummer genuegen. Die zweite ist deshalb erforder-
   lich, weil der verwendete poll-Auftrag parallel zu anderen Auftraegen
   im Server bearbeitet wird.
*/

 
/* Der in diesem Programm benutzte Datenbaustein ist immer derselbe.
   Daher definieren wir folgende Konstanten:
*/ 


/* =================================================== */
/*    Verbindung zu Simatic aufbauen                   */
/* =================================================== */
/*
   returns error code or 
   1111 if successful.

*/

int simatic_open(char *rc_file)

{
  int i;
  char hstr[300];

printf("simatic_open ..GESTARTET\n"); fflush(stdout);
printf("rc-file =%s\n",rc_file); fflush(stdout);

/* Programm Start */
  printf( "RK*-Client %s '%s' gestartet.\n\n",rk7_version(hstr),rc_file); fflush(stdout);

/*
	Bereitstellung / Ermittlung des Namens der Konfigurationsdatei
	--------------------------------------------------------------
	(gedacht als Beispiel, so macht es der Server )
	Falls genau ein Parameter uebergeben wurde, so nehmen wir
	an, dass es sich um den Namen der Konfigurationsdatei handelt.
	Anderfalls setzen wir die Voreinstellung.
*/

   printf("verbindung zu shared memory oeffnen ...\n");

 /* 
   rk7_open_shm wird mit 2 Parametern versorgt:
   - rc_file ist der weiter oben eingestellte Name der
   		Konfigurationsdatei
   - delaytime 60 Sekunden
   		Der Server ueberwacht den Zugriff des Clients auf seine
   		Auftraege. Wenn der Client Auftraege geoeffnet hat, so muss
   		er innerhalb der delaytime mindestens einen davon
   		benutzen, sonst schreitet der Server ein:
   		Alle Auftraege des Clients werden freigegeben.

  */
   if ((i =rk7_open_shm(rc_file,6000)) !=0) {
     printf ("%s\n", rk7_errortext(i));
     online = 0;
     return i;
   } else {
     online = 1;
   }

/*
Anmelden des von allen Ups verwendeten Auftrags beim Server.
-------------------------------------------------------------
	rk7_open_task reserviert einen Speicherbereich beim Server,
	der einen Puffer fuer die Daten sowie dazugehoerige
	Verwaltungsinformationen beinhaltet. Ohne einen solchen
	Speicherbereich (=Auftrag) kann kein Datentransfer
	durchgefuehrt werden.
	Die zurueckgelieferte Auftragsnummer liegt zwischen 0 und 9
	einschliesslich, falls die Reservierung von Speicher erfolgreich
	war. Sie ist bei den Transferaufrufen und beim Abmelden
	als Parameter anzugeben.
*/
  printf("Auftrag fuer Lesen oeffnen  ...\n");
  auftr_nr_f =rk7_open_task(400);
  if (auftr_nr_f < 0) {
     printf ("Fehler %d in rk7_open_task\n", auftr_nr_f);
     rk7_close_shm(40);
     online = 0;
     return auftr_nr_f;
  }
  printf ("Auftragsnummer %i erhalten.\n", auftr_nr_f);
 
  printf("Auftrag fuer Schreiben oeffnen  ...\n");
  auftr_nr_wr = rk7_open_task(400);
  if (auftr_nr_wr < 0) {
     printf ("Fehler %d in rk7_open_task\n", auftr_nr_wr);
     rk7_close_shm(40);
     online = 0;
     auftr_nr_f = auftr_nr_wr;
     return auftr_nr_wr;
  }
  printf ("Auftragsnummer %i erhalten.\n", auftr_nr_wr); 

  return 1111;
}

/* =================================================== */
/*     Verbindung zu Simatic abbauen                   */
/* =================================================== */

void simatic_close(void) 

{
  signed char function;
  int retval;

  if (online == 1) {
    /* Abmelden des zum Lesen verwendeten Auftrags. */
    printf ("Auftrag abmelden.\n");
    while ( /* Warten bis Auftrag beendet */
       (( retval = rk7_control(auftr_nr_f,&function)) != RK_IDLE)
       ) {
      usleep(100000L); /* Wartezeit */
      printf ("retval = %i\n",retval);
    } 
    rk7_close_task (auftr_nr_f, 10); /* 10 sekunden */

    /* Abmelden des zum Schreiben verwendeten Auftrags. */
    while (  /* Warten bis Auftrag beendet */
      ( (retval = rk7_control(auftr_nr_wr,&function)) != RK_IDLE) 
        && ( retval > -4) ) {
      usleep(100000L); /* Wartezeit */
      printf ("retval = %i\n",retval);
    } 
    rk7_close_task (auftr_nr_wr, 10); /* 10 sekunden */

    /* Abmelden des Shared memory, d.h. die Verbindung zum Server
    wird beendet. */
    printf ("Simatic abmelden.\n");
    rk7_close_shm (4); /* shared memory wieder abmelden */
    printf ("Simatic abmelden fertig\n");
    online = 0;
  }
}


/* =================================================== */
/*     control2 fuer Schreibauftrag                    */
/*     control2 for write-job                          */
/*          (check ob IDLE )                           */
/* =================================================== */
/* Return: 1 falls fertig, 0 falls aktiv        1 = ready, 0 = active 
           4 falls frei fuer Auftraege;         4 = idle; 

/* ldisp zeigt letzten Kopplungsstatus:         ldisp show the last link-stat 
     0 = OK, != 0 bei Fehler                       0 = OK, != 0 if error   
*/

int write_ready (void)

{
  signed char function;
  short wstate;

    /* pruefen, ob Auftrag nicht aktiv         check if job not active 
    */
    wstate = rk7_control(auftr_nr_wr,&function); /* Wartezeit in ms */

    /* da control2() die Stati nur beim Ende   Because control2() shows the state at end of transmission 
    des Auftrags fuer die Dauer eines          only for one call, the state has to be saved 
    Aufrufs anzeigt, muss der Status genau     in this moment.
    zu diesem Zeitpunkt gelesen werden. 
    */
    return wstate;
}


/* ======================================================================== */
/*     write-Anstoss fuer Schreibauftrag  (Kommandos an Simatic)            */
/*     start for    write-job             (commands to Simatic)             */
/*          (check ob IDLE )                                                */
/* ======================================================================== */
/* Return: 0 = ohne fehler,                     0 = no error
           != 0 error, see rk512_write_strt(); 

   ldisp zeigt letzten Kopplungsstatus:         ldisp show the last link-stat 
     0 = OK, != 0 bei Fehler                       0 = OK, != 0 if error   

   unsigned short *s5db  = array of unsigned short, representation of S5/S7-DB 
   char dbtype           = 'D' for DB and 'X'for DX
   unsigned short dbnr   = Nr of DB/DX
   int dwnr              = Nr of the first Word to handle
   int dwanz             = count of words to read
*/

int write_command( unsigned short *s5db,char db_type,int cpu, unsigned short dbnr, \
                   int bytenr, int bytecnt)

{ 
 int j;

 j = rk7_write_prio_strt(auftr_nr_wr,(void *)s5db,\
          db_type,dbnr,bytenr,bytecnt,\
          cpu,4000);
  if (j <= RK_AUFTR_LOST) { /* Abbruch der Verbindung       interrupt of link */
          online = 0;
  }
  return j;
}

/* ======================================================================== */
/*     lese-Anstoss fuer Schreibauftrag  (Kommandos an Simatic)             */
/*     ( nur benutzt bie direct-zugriff, Bitzustand lesen )                 */
/*     reading start for    write-job    (commands to Simatic)              */
/*     ( only used on direct access, read the bits state)                   */
/* ======================================================================== */
/* Return: 0 = ohne fehler,                     0 = no error
           != 0 error, see rk512_fetch_strt(); 

   ldisp zeigt letzten Kopplungsstatus:         ldisp show the last link-stat 
     0 = OK, != 0 bei Fehler                       0 = OK, != 0 if error   

   unsigned short *s5db  = array of unsigned short, representation of S5/S7-DB 
   char dbtype           = 'D' for DB and 'X'for DX
   unsigned short dbnr   = Nr of DB/DX
   int dwnr              = Nr of the first Word to handle
   int dwanz             = count of words to read
*/

int fetch_command( unsigned short *s5db,char db_type,int cpu, unsigned short dbnr, \
                   int bytenr, int bytecnt)

{
 int j;


 j = rk7_fetch_prio_strt(auftr_nr_wr,(void *)s5db,\
          db_type,dbnr,bytenr,bytecnt,\
          cpu,4000); /* Koppelmerker OFF = 255,15, CPU = 0 
                                                  timeout(msec) = 4000
                                               state 3964R, state rk512 */
  if (j <= RK_AUFTR_LOST) { /* Abbruch der Verbindung       interrupt of link */
          online = 0;
  }
  return j;
}




/* =================================================== */
/*     rk7_control fuer Fetch-Auftrag                  */
/*     rk7_control for fetch-job                       */
/*          (check if IDLE )                           */
/* =================================================== */
/* Return: 1 falls fertig, 0 falls aktiv        1 = ready, 0 = active */
/* ldisp zeigt letzten Kopplungsstatus:         ldisp show the last link-stat 
     0 = OK, != 0 bei Fehler                       0 = OK, != 0 if error   
*/

int fetch_ready (void)

{
  signed char function;
  short wstate = 4;
 
  if (online) {
    /* pruefen, ob Auftrag nicht aktiv         check if job not active 
    */
    wstate = rk7_control(auftr_nr_f,&function); /* Wartezeit in ms */

    /* da control() die Stati nur beim Ende   Because control2() shows the state at end of transmission 
    des Auftrags fuer die Dauer eines          only for one call, the state has to be saved 
    Aufrufs anzeigt, muss der Status genau     in this moment.
    zu diesem Zeitpunkt gelesen werden. 
    */
  }
  // printf("rk7_control ,ret = %i\n",wstate); 
  return wstate;
}

/* ======================================================================== */
/*     fetch-Anstoss fuer Schreibauftrag  (Prozessabbild von Simatic)       */
/*     start of fetch-job                 (process image from Simatic)      */
/*                                                                          */
/* ======================================================================== */
/* Return: 0 = ohne fehler,                     0 = no error
           != 0 error, see rk512_fetch_strt(); 

   ldisp zeigt letzten Kopplungsstatus:         ldisp show the last link-stat 
     0 = OK, != 0 bei Fehler                       0 = OK, != 0 if error   

   unsigned short *s5db  = array of unsigned short, representation of S5/S7-DB 
   char dbtype           = 'D' for DB and 'X'for DX
   unsigned short dbnr   = Nr of DB/DX
   int dwnr              = Nr of the first Word to handle
   int dwanz             = count of words to read
*/

int fetch_start( unsigned short *s5db,char db_type, unsigned short dbnr, \
                   int dwnr, int dwanz,int cpu)

{
  int j;

  j = rk7_fetch_strt(auftr_nr_f,(void *)s5db,\
          db_type,dbnr,dwnr *2 ,dwanz *2 ,\
          cpu,4000);

  /* printf("fetch_start ,ret = %i, stat=%i,s3964=%i\n",j,auftr_stat,stat512); */   
  return j;
}




