/* ====================================================================
   Rechnerkopplung zu SIMATIC-S5/S7  mit Prozedur RK512 / RK511
     am Beispiel von Pult, einer in xforms geschriebenen Simatic-Bedienoberflaeche

File: simatic_link.h

 (C)  Heisch Automation
      Im Vorderen Grosstal 4
      76857 Albersweiler
 und
      Gerald Emig
      Auf der Steige 7
      66564 Ottweiler/Fuerth


   =====================================================================
   Pult 2 Version 0.1
   10.04.2009  HW Umbau der API auf S7-Kompatibel

   19.02.2008  HW   simatic_link.h eingefuegt, Variablen fuer Anwenderprogramm 
                    verfuegbar gemacht
   Stand 0.2
   17.07.2000 HW fetch_ready() gibt im Fehlerfall Fehlerkennung aus, 
              statt idle ( vorheriger work around.
   20.06.2000 Fehleranzeige, wenn server nicht laeuft 
   06.05.2000

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>

#include "rk7_client.h"


/* =================================================== */
/* Datenbereich Global */


/* Datenbereich */
/* Zuerst werden die Speicherplaetze fuer die Auftragsnummern definiert.
   Grundsaetzlich koennen die Auftragsnummern abwechslend fuer
   Schreib- und Leseauftraege benutzt werden. Hauptsache ist, dass
   die Datenpakete in den Datenbereich, der im Server der Auftragsnummer
   zugeordnet ist, auch alle hineinpassen.
   Normalerweise wuerde fuer sequentielle Write- und Fetch-Auftraege somit
   eine einzige Auftragsnummer genuegen. Die zweite ist deshalb erforder-
   lich, weil der verwendete poll-Auftrag parallel zu anderen Auftraegen
   im Server bearbeitet wird.
*/


int online;                     /* 1 = Verbindung zu Simatic, 0 = keine Verbindung */  
int simatic_unlink_req;         /* request to unlink from simatic */

#define DB_SIZE 256
#define RK_IDLE 4               /* Auftrag ist frei */
#define RK_AUFTR_LOST -13       /* Auftrag gehoert nicht mehr zu dieser PID */
#define RK_SERVERDOWN -100      /* Server wurde beendet */


// #define RK_TIMEOUT 4000	/* Zeitlimit fuer Client-Server-Kommunikation */


/* =================================================== */
/*    Verbindung zu Simatic aufbauen                   */
/* =================================================== */
/* return 0 = OK, verbindung installiert,
          -1 = keine Verbindung */
int simatic_open(char *rc_file) ;


/* =================================================== */
/*     Verbindung zu Simatic abbauen                   */
/* =================================================== */

void simatic_close(void) ;



/* =================================================== */
/*     control2 fuer Schreibauftrag                    */
/*     control2 for write-job                          */
/*          (check ob IDLE )                           */
/* =================================================== */
/* Return: 1 falls fertig, 0 falls aktiv        1 = ready, 0 = active 
           4 falls frei fuer Auftraege;         4 = idle; 
*/

int write_ready ( void ) ;

/* ======================================================================== */
/*     write-Anstoss fuer Schreibauftrag  (Kommandos an Simatic)            */
/*     start for    write-job             (commands to Simatic)             */
/*          (check ob IDLE )                                                */
/* ======================================================================== */
/* Return: 0 = ohne fehler,                     0 = no error
           != 0 error, see rk512_write_strt(); 

   ldisp zeigt letzten Kopplungsstatus:         ldisp show the last link-stat 
     0 = OK, != 0 bei Fehler                       0 = OK, != 0 if error   

   unsigned short *s5db  = array of unsigned short, representation of S5/S7-DB 
   char dbtype           = 'D' for DB and 'X'for DX
   unsigned short dbnr   = Nr of DB/DX
   int bytenr            = Nr of the first byte to handle
   int bytecnt           = count of bytes to write
*/

int write_command( unsigned short *s5db,char db_type,int cpu, unsigned short dbnr, \
                   int bytenr, int bytecnt) ;

/* ======================================================================== */
/*     lese-Anstoss fuer Schreibauftrag  (Kommandos an Simatic)             */
/*     ( nur benutzt bie direct-zugriff, Bitzustand lesen )                 */
/*     reading start for    write-job    (commands to Simatic)              */
/*     ( only used on direct access, read the bits state)                   */
/* ======================================================================== */
/* Return: 0 = ohne fehler,                     0 = no error
           != 0 error, see rk512_fetch_strt(); 

   ldisp zeigt letzten Kopplungsstatus:         ldisp show the last link-stat 
     0 = OK, != 0 bei Fehler                       0 = OK, != 0 if error   

   unsigned short *s5db  = array of unsigned short, representation of S5/S7-DB 
   char dbtype           = 'D' for DB and 'X'for DX
   unsigned short dbnr   = Nr of DB/DX
   int bytenr            = Nr of the first byte to handle
   int bytecnt           = count of bytes to read
*/

int fetch_command( unsigned short *s5db,char db_type,int cpu, unsigned short dbnr, \
                    int bytenr, int bytecnt) ;


/* =================================================== */
/*     control2 fuer Fetch-Auftrag                     */
/*     control2 for fetch-job                          */
/*          (check ob IDLE )                           */
/* =================================================== */
/* Return: 1 falls fertig, 0 falls aktiv        1 = ready, 0 = active 
           4 = idle                             4 = idle
*/

int fetch_ready ( void) ;


/* ======================================================================== */
/*     fetch-Anstoss fuer Schreibauftrag  (Prozessabbild von Simatic)       */
/*     start of fetch-job                 (process image from Simatic)      */
/*                                                                          */
/* ======================================================================== */
/* Return: 0 = ohne fehler,                     0 = no error
           != 0 error, see rk512_fetch_strt(); 
    
   ldisp zeigt letzten Kopplungsstatus:         ldisp show the last link-stat 
     0 = OK, != 0 bei Fehler                       0 = OK, != 0 if error   

   unsigned short *s5db  = array of unsigned short, representation of S5/S7-DB 
   char dbtype           = 'D' for DB and 'X'for DX
   unsigned short dbnr   = Nr of DB/DX
   int bytenr            = Nr of the first byte to handle
   int bytecnt           = count of bytes to read
*/

int fetch_start( unsigned short *s5db,char db_type, unsigned short dbnr, \
                   int bytenr, int bytecnt,int cpu) ;


