/* ================================================================
   routinen zur String-Verarbeitung und Ueberpruefung

   (c) Heisch Automatisierungstechnik, Werner Heisch
       hw@heisch.inka.de
       http://sites.inka.de/heisch

   ================================================================
   Stand 20/09/1998
         16/06/2001
         21/06/2001  is_int,is_float,sstrtok
         05/02/2003 ctokenize() neu!
         22/12/2003 hwdate_fr_slash() neu
         01/04/2004 in isint und isfloat ( Vorzeiche +,- eingebaut
         24.09.2004 sstrcat() eingebaut
         08.10.2004 sep_dir_finam() eingebaut
         10.04.2005 char *sstrcat immer return (dest)
         26.12.2006  isint, isfloat ekzeptiert fuehrende blanks

         26.01.2009 sep_dir_finam() geendert: Rueckgabe fi, auch wenn kein pfad,
         27.01.2009 tokenize() : leerer Parameter oignpos1 entfernt
	 10.02.2014 *trimm_str_alnum() eingebaut
*/

#include <stdio.h>
#include <string.h>
#include "stringc.h"

#ifndef HSTR_LEN
#define HSTR_LEN 16384
#endif

/* ----------- Ueberpruefung eines Strings ob integer-Zahl --------- */
/* Ergebnis: 1 = integer, 0 = keine integer */
 
int isint(const char s[] )

{
  int i,len;

  if ((len = strlen(s)) == 0)
    return 0;
  i = 0;
  while ( (i<len) && (s[i] == ' '))
     ++i;
  if (i >=len) return 0;
  if ((s[i] == '-') || (s[i] == '+')) {
    i ++ ;
    if (len < i)
      return 0;
  }
  while ( (i<len) && isdigit(s[i]))
     ++i;
  return !(len - i);
}

/* ----------- Ueberpruefung eines Strings ob float-Zahl --------- */
/* Ergebnis: 1 = float, 0 = kein float  
   ( eine Integerzahl ist auch eine float )
*/
int isfloat(const char s[] )

{ 
  int i,len;

  if ((len = strlen(s)) == 0) {
    return 0;
  }
  i = 0;
  while ( (i<len) && (s[i] == ' '))
     ++i;
  if (i >=len) return 0;
  if ((s[i] == '-') || (s[i] == '+')) {
    i = 1;
    if (len < i)
      return 0;
  }
  while ( (i<len) && isdigit(s[i]))
     i++;
  if ( i < (len -1 )) {
    if (s[i] != '.') {
      return 0;
    }
    else i++;
  }  
  while ( (i < len) && isdigit(s[i])) 
     i++;
  return !(len -i);
}

/* ----------- Ueberpruefung eines Strings ob Hex-Zahl --------- */
/* xxxxx .. Wobei x in ('0Ä..'9' oder 'A'..'F' oder 'a' ..'f' */
/* Ergebnis: <> 0 = hex 0 = kein hex */
 
int ishex(const char s[] )

{
  int i,len;

  if ((len = strlen(s)) == 0)
    return 0;
  i = 0;
  while ( (i<len) && (s[i] == ' '))
     ++i;
  if (i >=len) return 0;
  while ( (i<len) && isxdigit(s[i]))
     ++i;
  return !(len - i);
}


/* ----------- Ueberpruefung eines Strings ob datum, deutsches Format ---- */
/* String = [d]d.[m]m.[yy]yy, Jahr 2- oder 4-stellig.
  Ergebnis: 2,4 (Stellen der Jahre) = datum, 0 = kein datum
            -1 : String zu lang oder zu kurz, nicht interpretierbar.
             0 : kein Datumsformat in [d]d.[m]m.[yy]yy -notation.
  wenn ein gueltiges datum vorliegt und sd[] != NULL dann wird das
  Ergebnis in HW-Zeitformat yyyymmdd zurueckgegeben
*/

int hw_datestr(char ds[],char s[])

{
  int i,j,len;
  int jahr,monat,tag;

  // printf("  !!!! hw_datestr >%s<...",s);
  jahr = monat = tag = 0;
  if ((len = strlen(s)) > 11) return -1;
  i = 0;
  j = 0;
  while ((i<len) && (( j = s[i] -'0') >=0) && (j <=9)) {
    tag = tag *10 + j;
    i++;
  }
  // printf(" tag=%d",tag);
  if (( i >= len) || (s[i] != '.')) return 0;
  i++;
  j = 0;
  while ((i<len) && (( j = s[i] -'0') >=0) && (j <=9)) {
    monat = monat *10 + j;
    i++;
  }
  // printf(" monat=%d",monat);
  if (( i >= len) || (s[i] != '.')) return 0;
  i++;
  j = 0;
  while ((i<len) && (( j = s[i] -'0') >=0) && (j <=9)) {
    jahr = jahr *10 + j;
    i++;
  }
  // printf(" jahr=%d",jahr);

  if ((tag<1) || (tag>31) || (monat<1) || (monat>12) || (jahr<0))
    return 0;
  if (jahr<1899) {
    if (jahr > 50) {
      jahr += 1900;
    } else {
      jahr += 2000;
    }
    j = 2;
  } else {
    j =4;
  }
  if ( ds != NULL)
    sprintf(ds,"%04d%02d%02d",jahr,monat,tag);
  return j;
}

/* ----------- Ueberpruefung eines Strings ob datum mit / getrennt, deutsches Format ---- */
/* String = [d]d/[m]m/[yy]yy, Jahr 2- oder 4-stellig.
  Ergebnis: 2,4 (Stellen der Jahre) = datum, 0 = kein datum
            -1 : String zu lang oder zu kurz, nicht interpretierbar
             0 : kein Datumsformat in [d]d/[m]m/[yy]yy -notation.
  wenn ein gueltiges datum vorliegt und sd[] != NULL dann wird das
  Ergebnis in HW-Zeitformat yyyymmdd zurueckgegeben
  Wenn Jahreszahl > 50, dann 19xx, sonst 20xx
*/

int hw_datestr_fr_slash(char ds[],char s[])

{
  int i,j,len;
  int jahr,monat,tag;

  // printf("  !!!! hw_datestr >%s<...",s);
  jahr = monat = tag = 0;
  if ((len = strlen(s)) > 11) return -1;
  i = 0;
  j = 0;
  while ((i<len) && (( j = s[i] -'0') >=0) && (j <=9)) {
    tag = tag *10 + j;
    i++;
  }
  // printf(" tag=%d",tag);
  if (( i >= len) || (s[i] != '/')) return 0;
  i++;
  j = 0;
  while ((i<len) && (( j = s[i] -'0') >=0) && (j <=9)) {
    monat = monat *10 + j;
    i++;
  }
  // printf(" monat=%d",monat);
  if (( i >= len) || (s[i] != '/')) return 0;
  i++;
  j = 0;
  while ((i<len) && (( j = s[i] -'0') >=0) && (j <=9)) {
    jahr = jahr *10 + j;
    i++;
  }
  // printf(" jahr=%d",jahr);

  if ((tag<1) || (tag>31) || (monat<1) || (monat>12) || (jahr<0))
    return 0;
  if (jahr<1899) {
    if (jahr > 50) {
      jahr += 1900;
    } else {
      jahr += 2000;
    }
    j = 2;
  } else {
    j =4;
  }
  if ( ds != NULL)
    sprintf(ds,"%04d%02d%02d",jahr,monat,tag);
  return j;
}

/* ---------------------- */
/* tokenisiere Eingabestring  (intern benutzt: strtok() */
/* Eingabe: buf: String
     tok 2-dimensionales Character-array = array[tanz] of string[tlen];
     tok[tanz][tlen]
   Ausgabe:
     tok gefuellt,
     returnwert = Anzahl der gueltigen tokens;

   Diese function manipuliert nicht den Eingabestring buf[] !!

*/

int tokenize( char buf[],char tokseps[], char toks[],int tanz,int tlen)

{

char hstr[HSTR_LEN];
char *tptr = NULL;
char *sptr = NULL;
char *zptr = NULL;
int anz = 0;


  if ( strlen(buf) > 0) {
    strcpy(hstr,buf);
    /*
      printf("hstr = >%s<",hstr);
      printf("zptr = %x\n", zptr);
    */
    zptr = toks;
    /* 
    printf("zptr = %x\n", zptr);
    */
    sptr = hstr;

    while (( anz < tanz) && ((tptr = (char *)strtok(sptr,tokseps)) != NULL)) { 
      /* 
        strcpy(tstr,tptr);
        printf("%s\n",tstr);
      */  
      strcpy(zptr, tptr); 
      /* 
        printf("%x ==>%s<==\n",zptr,zptr);
      */
      zptr= zptr + tlen;
      anz ++;
      sptr = NULL;
    }
  }
  /*
  printf("tokenize ende\n"); fflush(stdout);
  */
  return(anz);
}

/* ---------------------- */
/* tokenisiere Eingabestring */
/* Eingabe: buf: String
     tok 2-dimensionales Character-array = array[tanz] of string[tlen];
     tok[tanz][tlen]
   Ausgabe:
     tok gefuellt,
     returnwert = Anzahl der gueltigen tokens;              
*/

int tokenize_sview_alt( char buf[], char toks[],int tanz,int tlen)

{
char tstr[255];
char hstr[1024];
char tok[1024];
char *tptr = NULL;
char *sptr = NULL;
char *zptr = NULL;
int anz = 0;
char tokseps[] = "\t \n";
 
  if ( strlen(buf) > 0) {
    strcpy(hstr,buf);
    /* printf("hstr = >%s<",hstr);
    printf("zptr = %x\n", zptr);
    */
    zptr = toks;
    /* 
    printf("zptr = %x\n", zptr);
    */
    sptr = hstr;
    while (( anz < tanz) && (( tptr = (char *)strtok(sptr,tokseps)) != NULL)) {
      /* 
      strcpy(tstr,tptr);
      printf("%s\n",tstr);
      */
      strcpy(zptr, tptr); 
      /* 
       printf("%x ==>%s<==\n",zptr,zptr);
      */
      zptr= zptr + tlen;
      anz++;
      sptr = NULL;
    }  
  } 
  /* 
  printf("tokenize ende\n"); fflush(stdout);
  */
  return(anz);
}

/* ---------------------------------- */
/* Umsetzen eines Strings in Grossbuchstaben
*/
char *str_toupper(char *s)

{
  size_t i = 0;

  while(s[i] != '\0') {
    s[i] = toupper(s[i]);
    i ++ ; 
  }
  return s;
}

/* ---------------------------------- */
/* Umsetzen eines Strings in Kleinbuchstaben
*/
char *str_tolower(char *s)

{
  size_t i = 0;

  while(s[i] != '\0') {
    s[i] = tolower(s[i]);
    i ++ ; 
  }
  return s;
}

/* ---------------------------------- */ 
/* String aufspalten in Teilstrings
   ( angelehnt an strtok(), Erweiterung, 
      manipuliert aber nicht den
     Quellstring ( siehe man strtok(3) )


   diese function unterstuetzt nur quellstrings bis zu HSTR_LEN zeichen)

   bei *delim - stringlaenge 0 ( = leer ) wird der gesamte Rest uebergeben
   opt : options hier koennen Optionen uebergeben werden.
        zulaessige  Optionen sind:
        's' = ignoriere Delimiter am Anfang des Suchstrings, Start beim
              ersten nicht-delimiter-Zeichen
        'c' = Betrachte eine Sequenz, eingeschlossen in '"' als ein einziges Element,
              die darin liegenden delimiter werden ignoriert. 
              !!! das Zeichen '"' darf dann kein delimiter sein.

   der gefundene String wird nach d kopiert,
   falls nichts gefunden wurde, wird "" in d kopiert
   es wird geprueft, ob d lang genug ist (dlen)
   retval: -1 : in Suchstring wurde Ende erreicht;
           -2 : bei Suchstring initialisierung *s != NULL: *s zu lang
           -3 : gefunden, aber *d ist zu klein fuer gefundenes 
           alles OK: Token aus delim, der gefunden wurde
                     ggf. auch 0, wenn bis Ende gelesen
*/

char cstrtok(char *d,unsigned int dlen, const char *s, const char *delim,const char *opt) 

{
  static char hstr[HSTR_LEN] = "";
  static int strtidx = 0;
  static int length = 0;

  int found = 0;

  int i;  /* Laufindex */
  int j;  /* laufindex in delim */
  char *strptr;
  int stripp_strt = 0;         /* ignore delimiters at the beginning */
  int strcomplete = 0;         /* return a string complete, if set into paranthesis */ 
  signed char in_str = 0;             /* in a sequenze beginning with " .. */ 
  signed char found_delim = 0;

  if (s != NULL ) { /* wird neu besetzt */
    if ( strlen(s) > (sizeof(hstr) -1)) { /*Quellstring ist zu lang */
      strcpy(hstr,"");
      strtidx = 0;
      length = 0;
      strcpy(d,"");  /* Zielstring leer besetzen */
      return -2;
    } else {  /* Laenge ist OK, initialisiere hstr */
      strcpy(hstr,s);
      strtidx = 0;
      length = strlen(hstr);
    }
  }
  /* Suchen des tokens */
  if (strtidx >= length) { /* Stringende erreicht */
    strcpy(d,"");  /* Zielstring leer besetzen */
    return -1;
  }

  i = strtidx ;

  /* durchsuchen der options */  
  if (strlen(opt) >=1 ) {
    if (strchr(opt,'s') != 0)
      stripp_strt = 1;         /* ignore delimiters at the beginning */

    if (strchr(opt,'c') != 0)
      strcomplete = 1;         /* return a string complete, if set into paranthesis */ 
  }
  /* Sonderfunktion */
  /* gesamten Rest holen ? */
  if (strlen(delim) == 0) {
    /* gesamten Rest holen */
    i = strlen(hstr);
    found = 1;
    found_delim = 0; /* Stringende erreicht */
  } else {
    /* delimiter am Anfang verwerfen ? */
    if (stripp_strt) {
      found = 1;
      while ((hstr[i] != '\0' ) && (found==1)) { /* Laufen in hstr */ 
        found = 0;
        for (j=0; (( delim[j] != '\0') && (found ==0)); j++ ) { /* laufen in delim */
          if( hstr[i] == delim[j] ) { /* delimiter gefunden */
            strtidx = i = i + 1;
            found = 1;
          }
        }
      }
    }
    /* delimiter suchen */
    while((hstr[i] != '\0' ) && (found==0)) { /* Laufen in hstr */
      if((strcomplete) && (hstr[i] == '"'))
        in_str = ~in_str;
      if(!in_str) {
        for (j=0; (( delim[j] != '\0') && (found ==0)); j++ ) { /* laufen in delim */
          if( hstr[i] == delim[j] ) { /* delimiter gefunden */
            found = 1;
            found_delim = delim[j]; /* dieser Delimiter wurde gefunden */
          }
        }
      }
      if (found == 1) break;
      i ++;
    }
  } 
  if (hstr[i] == '\0') { /* Stringende erreicht = Gefunden */
    found = 1;
    found_delim = 0; /* Stringende erreicht */
  } 
  if (found == 1) { /* es wurde was gefunden */
    hstr[i] = '\0'; /* String abschliessen */
    strptr = &hstr[strtidx];
    if (strlen(strptr) < dlen) {
      strcpy(d,strptr);  /* Zielstring mit gefundenem String besetzen */
    } else {  /* falls zielstring zu kurz */
      strcpy(d,"");  /* Zielstring leer besetzen */
      found_delim = -3; /* Zielstring ist zu kurz */
    }
    strtidx = i+1;  /* zeiger hinter neues Stringende */
    return found_delim;
  } 
  /* nichts gefunden */
  strtidx = i;
  strcpy(d,"");  /* Zielstring leer besetzen */
  return 0;
}

/* ---------------------- */
/* tokenisiere Eingabestring  */
/* Eingabe: buf: String
     toks 2-dimensionales Character-array = array[tanz] of string[tlen];
     toks[tanz][tlen]
   Ausgabe:
     toks gefuellt,
     returnwert = Anzahl der gueltigen tokens;

Unterschied zu tokenize():
Falls 2 Delimiter direkt auf einander folgen, wird ein Leerstring erzeugt.
 tokenize(), weil strtok basiert, erkennt in diesem Fall nur jeden 2. Delimiter.


   Diese function manipuliert nicht den Eingabestring buf[] !!
   opt : options hier koennen Optionen uebergeben werden.
        zulaessige  Optionen sind:
        's' = ignoriere Delimiter am Anfang des Suchstrings, Start beim
              ersten nicht-delimiter-Zeichen
        'c' = Betrachte eine Sequenz, eingeschlossen in '"' als ein einziges Element,
              die darin liegenden delimiter werden ignoriert. 
              !!! das Zeichen '"' darf dann kein delimiter sein.

   zusaetzliche Funktionalitaet:
      Wenn 2 Delimiter aufeinander folgen, so wird ein Leerstring 
      zurueckgegeben.
*/

int ctokenize( const char buf[],char tokseps[], char *toks,int tanz,int tlen,const char *opt)

{

char hstr[HSTR_LEN];
char *sptr = NULL;
char *zptr = NULL;
int anz = 0;
int cstrtok_erg;

  if ( strlen(buf) > 0) {
    strcpy(hstr,buf);
    /*
      printf("hstr = >%s<",hstr);
      printf("zptr = %x\n", zptr);
    */
    zptr = toks;
    /*
    printf("zptr = %x\n", zptr);
    */
    sptr = hstr;

    do {
      if ((cstrtok_erg = cstrtok(zptr,tlen, sptr,tokseps,opt)) >=0 )
      /* strcpy(zptr, tptr); */ 
      /* 
        printf("%x ==>%s<==\n",zptr,zptr);
      */
      zptr= zptr + tlen;
      anz ++;
      sptr = NULL;


    }  while (( anz < tanz) && (cstrtok_erg > -1)) ; 
  } 
  /* 
  printf("tokenize ende\n"); fflush(stdout);
  */
  return(anz-1);
}

/* ****************************************** */
/* Alle letzte Zeichen 'c' eines Strings loeschen
   z.B. ' ' loescht alle blancs am Stringende 
*/
char *trimm_str(char s[],char c)

{
  int i = strlen(s);
  while (( i >= 1) && (s[i-1] == c)) {
    s[i-1] = '\0';
    i -- ;
  }    
  return s;
}

/* ****************************************** */
/* Alle letzte Zeichen eines Strings die nicht 
   sichtbar sind, inclusive <space> sind, löschen
   z.B. <cr> <lf><tab> 
*/
char *trimm_str_visible(char s[])

{
  int i = strlen(s);

  while (( i >= 1) && (!isprint(s[i-1]) || (s[i-1] == '\r')
   || (s[i-1] == '\n') || (s[i-1] == '\t')
   || isspace(s[i-1])) ) {
    s[i-1] = '\0';
    i -- ;
  }
  return s;
}


/* ****************************************** */
/* sicheres String-Copy:
   Es werden maximal (n-1) byte kopiert, der 
   Zielstring wird mit '\0' abgeschlossen

Es erfolgt keine Fehlermeldung, wenn der Zielstring zu 
kurz ist.  

*/
char *sstrcpy(char *dest, const char *src, size_t n)

{
  int l = strlen(src); 
  if ( l < n) { /* Ziel passt in Quelle */
    memcpy(dest,src,l+1);
  } else { /* Ziel passt nicht in Quelle */
    memcpy(dest,src,(n-1));
    *(dest+l) = '\0'; /* String mit 0 abschliessen */
  }
  return (dest);
}

/* ****************************************** */
/* sicheres String-cat:
   Es werden maximal (n-1) byte kopiert, der 
   Zielstring wird mit '\0' abgeschlossen

Der Zielsting muss initialisiert sein !

Es erfolgt keine Fehlermeldung, wenn der Zielstring zu 
kurz ist.

*/
char *sstrcat(char *dest, const char *src, size_t n)

{
  int l = strlen(src);
  int dl = strlen(dest);
  int res = n - dl ; /* Rest-platz in Ziel */
  if (dl > n)
    return (dest); /* dest vermutlich nicht initialisiert, garnix tun */
  if ( l < res) { /* Ziel passt in Quelle */
     memcpy((dest+dl),src,l+1);
  } else { /* Ziel passt nicht in Quelle */
    memcpy((dest+dl),src,(res-1));
    *(dest+dl+l) = '\0'; /* String mit 0 abschliessen */
  }
  return (dest); 
}

/* --------------------------- */
/* Ausgabe eines Datenbereichs als hex-dump */

void print_buf (char b[],int len)

{
  int i;
  unsigned char c;
  for (i=0; i<len;i++){
    c = b[i];
    printf("%02x ",c);
    if ((((i+1) % 16 ) == 0) && ( i != 0)) printf("\n");
  }
}


/* ---------------------------- */
/* Trennen des directory und Dateiname 

   retval > 0; Länge des Directory
          = 0; OK, keine Directory-Angabe
          = -1 dir-string zu klein
          = -2 file-string zu klein 

*/ 
int sep_dir_finam(char dir[],int dirlen, char fi[], int filen, const char lfi[])

{
  int i, max;
  int retval ;

  max = strlen(lfi); /* Gesamtlänge */
  retval = 0; /*Vorbesetzung */

  i = max - 1 ; /* Position letztes Char in lfi */

  while (( i > 0 ) && ( lfi[i] != '/' )) i -- ;

  /* i steht auf letztem '/' -Trennzeichen */
  if ( i > 0)  { /* Directory-Anteil */
    if ( dirlen > i) {
      strncpy(dir,lfi,i);
      dir[i] = '\0';
      retval = i;
    } else { 
      return -1; 
    }
  } else {
    dir[0] = '\0';
    retval = 0;
  }
  if (i > 0) i++ ;
  if (filen > (max - i) ) {
    strncpy(fi,&(lfi[i]),(max -i +1));
  } else  {
    return -2;
  }
  return 0; 
}


