/* ================================================================
   routinen zur String-Verarbeitung und Ueberpruefung

   (c) Heisch Automatisierungstechnik, Werner Heisch
       hw@heisch.inka.de
       http://sites.inka.de/heisch

   ================================================================
   Stand 20/09/1998
         16/06/2001
         21/06/2001  is_int,is_float,sstrtok
         05/02/2003 ctokenize() neu!
         22/12/2003 hwdate_fr_slash() neu
         01/04/2004 in isint und isfloat ( Vorzeiche +,- eingebaut
         24.09.2004 sstrcat() eingebaut
         08.10.2004 sep_dir_finam() eingebaut
         10.04.2005 char *sstrcat immer return (dest)
         26.12.2006  isint, isfloat ekzeptiert fuehrende blanks

         26.01.2009 sep_dir_finam() geendert: Rueckgabe fi, auch wenn kein pfad,
         27.01.2009 tokenize() : leerer Parameter oignpos1 entfernt
	 10.02.2014 *trimm_str_alnum() eingebaut
*/

#include <stdio.h>
#include <string.h>

#ifndef HSTR_LEN
#define HSTR_LEN 16384
#endif


/* ----------- Ueberpruefung eines Strings ob integer-Zahl --------- */
/* Ergebnis: 1 = integer, 0 = keine integer */
 
int isint( const char s[]);


/* ----------- Ueberpruefung eines Strings ob Hex-Zahl --------- */
/* xxxxx .. Wobei x in ('0Ä..'9' oder 'A'..'F' oder 'a' ..'f' */
/* Ergebnis: 1 = integer, 0 = keine integer */
 
int ishex(const char s[] ) ;

/* ----------- Ueberpruefung eines Strings ob float-Zahl --------- */
/* Ergebnis: 1 = float, 0 = kein float  
   ( eine Integerzahl ist auch eine float )
*/
int isfloat( const char s[] );


/* ----------- Ueberpruefung eines Strings ob datum, deutsches Format ---- */
/* String = [d]d.[m]m.[yy]yy, Jahr 2- oder 4-stellig.
  Ergebnis: 2,4 (Stellen der Jahre) = datum, 0 = kein datum  
            -1 : String zu lang oder zu kurz, nicht interpretierbar.
  wenn ein gueltiges datum vorliegt und sd[] != NULL dann wird das 
  Ergebnis in HW-Zeitformat yyyymmdd zurueckgegeben           
*/

int hw_datestr(char ds[],char s[]);

/* ----------- Ueberpruefung eines Strings ob datum mit / getrennt, deutsches Format ---- */
/* String = [d]d/[m]m/[yy]yy, Jahr 2- oder 4-stellig.
  Ergebnis: 2,4 (Stellen der Jahre) = datum, 0 = kein datum
            -1 : String zu lang oder zu kurz, nicht interpretierbar
             0 : kein Datumsformat in [d]d/[m]m/[yy]yy -notation.
  wenn ein gueltiges datum vorliegt und sd[] != NULL dann wird das
  Ergebnis in HW-Zeitformat yyyymmdd zurueckgegeben
  Wenn Jahreszahl > 50, dann 19xx, sonst 20xx
*/

int hw_datestr_fr_slash(char ds[],char s[]);

/* ---------------------- */
/* tokenisiere Eingabestring  (intern benutzt: strtok() */
/* Eingabe: buf: String
     tok 2-dimensionales Character-array = array[tanz] of string[tlen];
     tok[tanz][tlen]
   Ausgabe:
     tok gefuellt,
     returnwert = Anzahl der gueltigen tokens;              

   Diese function manipuliert nicht den Eingabestring buf[] !!

*/

int tokenize( char buf[],char tokseps[], char toks[],int tanz,int tlen);


/* ---------------------------------- */
/* Umsetzen eines Strings in Grossbuchstaben
*/
char *str_toupper(char *s);


/* ---------------------------------- */
/* Umsetzen eines Strings in Kleinbuchstaben
*/
char *str_tolower(char *s);


/* ---------------------------------- */ 
/* String aufspalten in Teilstrings
   ( angelehnt an strtok(), Erweiterung, 
      manipuliert aber nicht den
     Quellstring ( siehe man strtok(3) )


   diese function unterstuetzt nur quellstrings bis zu HSTR_LEN zeichen)

   bei *delim - stringlaenge 0 ( = leer ) wird der gesamte Rest uebergeben
   opt : options hier koennen Optionen uebergeben werden.
        zulaessige  Optionen sind:
        's' = ignoriere Delimiter am Anfang des Suchstrings, Start beim
              ersten nicht-delimiter-Zeichen
        'c' = Betrachte eine Sequenz, eingeschlossen in '"' als ein einziges Element,
              die darin liegenden delimiter werden ignoriert. 
              !!! das Zeichen '"' darf dann kein delimiter sein.

   der gefundene String wird nach d kopiert,
   falls nichts gefunden wurde, wird "" in d kopiert
   es wird geprueft, ob d lang genug ist (dlen)
   retval: -1 : in Suchstring wurde Ende erreicht;
           -2 : bei Suchstring initialisierung *s != NULL: *s zu lang
           -3 : gefunden, aber *d ist zu klein fuer gefundenes 
           alles OK: Token aus delim, der gefunden wurde
                     ggf. auch 0, wenn bis Ende gelesen
*/

char cstrtok(char *d,unsigned int dlen, const char *s, const char *delim,const char *opt);


/* ---------------------- */
/* tokenisiere Eingabestring  */
/* Eingabe: buf: String
     toks 2-dimensionales Character-array = array[tanz] of string[tlen];
     toks[tanz][tlen]
   Ausgabe:
     toks gefuellt,
     returnwert = Anzahl der gueltigen tokens;              

Unterschied zu tokenize():
Falls 2 Delimiter direkt auf einander folgen, wird ein Leerstring erzeugt.
 tokenize(), weil strtok basiert, erkennt in diesem Fall nur jeden 2. Delimiter.


   Diese function manipuliert nicht den Eingabestring buf[] !!
   opt : options hier koennen Optionen uebergeben werden.
        zulaessige  Optionen sind:
        's' = ignoriere Delimiter am Anfang des Suchstrings, Start beim
              ersten nicht-delimiter-Zeichen
        'c' = Betrachte eine Sequenz, eingeschlossen in '"' als ein einziges Element,
              die darin liegenden delimiter werden ignoriert. 
              !!! das Zeichen '"' darf dann kein delimiter sein.

   zusaetzliche Funktionalitaet:
      Wenn 2 Delimiter aufeinander folgen, so wird ein Leerstring 
      zurueckgegeben.
*/

int ctokenize( const char buf[],char tokseps[], char *toks,int tanz,int tlen,const char *opt);



/* ****************************************** */
/* Alle letzte Zeichen 'c' eines Strings loeschen
   z.B. ' ' loescht alle blancs am Stringende 
*/
char *trimm_str(char s[],char c);


/* ****************************************** */
/* Alle letzte Zeichen eines Strings die nicht 
   sichtbar sind, inclusive <space> sind, löschen
   z.B. <cr> <lf><tab> 
*/
char *trimm_str_visible(char s[]);

/* ****************************************** */
/* sicheres String-Copy:
   Es werden maximal (lin-1) byte kopiert, der 
   Zielstring wird mit '\0' abgeschlossen

Es erfolgt keine Fehlermeldung, wenn der Zielstring zu 
kurz ist.

*/
char *sstrcpy(char *dest, const char *src, size_t n);



/* ****************************************** */
/* sicheres String-cat:
   Es werden maximal (n-1) byte kopiert, der 
   Zielstring wird mit '\0' abgeschlossen

Der Zielsting muss initialisiert sein !

Es erfolgt keine Fehlermeldung, wenn der Zielstring zu 
kurz ist.  

*/
char *sstrcat(char *dest, const char *src, size_t n);


/* --------------------------- */
/* Ausgabe eines Datenbereichs als hex-dump */

void print_buf (char b[],int len) ;

/* ****************************************** */
/* Trennen des directory und Dateiname 

   retval > 0; Länge des Directory
          = 0; OK, keine Directory-Angabe
          = -1 dir-string zu klein
          = -2 file-string zu klein 



  Parameter Eingang: lfi : Quellfile
                     filen : Laenge des Strings fi
                     dirlen : Laenge des Strings dir

            Ausgang : dir : Directory-Anteil des ganzen Namens
                      fi  : Filename

*/
int sep_dir_finam(char dir[],int dirlen, char fi[], int filen, const char lfi[]) ;


